@@ -15,7 +15,7 @@ import docker
 import requests
 from jinja2 import Environment, FileSystemLoader
 
-from simplyblock_core import utils, scripts, constants, mgmt_node_ops, storage_node_ops, distr_controller
+from simplyblock_core import utils, scripts, constants, mgmt_node_ops, storage_node_ops, distr_controller, shell_utils
 from simplyblock_core.controllers import cluster_events, device_controller, storage_events, pool_controller, \
     lvol_controller
 from simplyblock_core.db_controller import DBController
@@ -33,6 +33,7 @@ from simplyblock_core.models.storage_node import StorageNode
 logger = logging.getLogger()
 TOP_DIR = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 
+db_controller = DBController()
 
 def _create_update_user(cluster_id, grafana_url, grafana_secret, user_secret, update_secret=False):
     session = requests.session()
@@ -202,8 +203,8 @@ def create_cluster(blk_size, page_size_in_blocks, cli_pass,
     if prov_cap_crit and prov_cap_crit > 0:
         c.prov_cap_crit = prov_cap_crit
     if distr_ndcs == 0 and distr_npcs == 0:
-        c.distr_ndcs = 1
-        c.distr_npcs = 1
+        logger.error("both distr_ndcs and distr_npcs cannot be 0")
+        return False
     else:
         c.distr_ndcs = distr_ndcs
         c.distr_npcs = distr_npcs
@@ -294,7 +295,222 @@ def create_cluster(blk_size, page_size_in_blocks, cli_pass,
     logger.info(c.uuid)
     return c.uuid
 
+def parse_nvme_list_output(output, target_model):
+    lines = output.splitlines()
+    for line in lines:
+        if target_model in line:
+            return line.split()[0]
+    raise Exception(f"Device with model {target_model} not found in nvme list")
 
+def cleanup_nvme(mount_point, nqn_value):
+    if not nqn_value:
+        logger.error("NQN value is empty. Skipping disconnect.")
+        return False
+    
+    logger.info(f"Starting cleanup for NVMe device with NQN: {nqn_value}")
+
+    # Unmount the filesystem
+    try:
+        subprocess.run(["sudo", "umount", mount_point], check=True)
+        logger.info(f"Unmounted {mount_point}")
+    except subprocess.CalledProcessError:
+        logger.warning(f"Failed to unmount {mount_point}, continuing...")
+        return False
+
+    # Disconnect NVMe device
+    try:
+        subprocess.run(["sudo", "nvme", "disconnect", "-n", nqn_value], check=True)
+        logger.info(f"Disconnected NVMe device: {nqn_value}")
+    except subprocess.CalledProcessError:
+        logger.error(f"Failed to disconnect NVMe device: {nqn_value}")
+        return False
+
+    # Remove the mount point directory
+    try:
+        subprocess.run(["sudo", "rm", "-rf", mount_point], check=True)
+        logger.info(f"Removed mount point: {mount_point}")
+    except subprocess.CalledProcessError:
+        logger.warning(f"Failed to remove mount point {mount_point}")
+        return False
+
+    return True
+
+def run_fio(mount_point):
+
+    if not mount_point:
+        raise ValueError("Mount point cannot be empty.")
+
+    if not os.path.exists(mount_point):
+        os.makedirs(mount_point, exist_ok=True)
+    
+    fio_config = f"""
+[test]
+ioengine=aiolib
+direct=1
+iodepth=4
+readwrite=randrw
+bs=4K
+nrfiles=4
+size=1G
+verify=md5
+numjobs=3
+directory={mount_point}
+"""
+    config_file = "fio.cfg"
+    with open(config_file, "w") as f:
+        f.write(fio_config)
+    
+    try:
+        result = subprocess.run(["sudo", "fio", config_file], check=True, text=True, capture_output=True)
+        logger.info(result.stdout)
+    except subprocess.CalledProcessError as e:
+        logger.error(f"Error running fio: {e.stderr}")
+    finally:
+        if os.path.exists(config_file):
+            os.remove(config_file)
+            logger.info("fio configuration file removed.")
+    
+
+def deploy_cluster(storage_nodes,test,ha_type,distr_ndcs,distr_npcs,enable_qos,ifname,blk_size, page_size_in_blocks, cli_pass,
+                   cap_warn, cap_crit, prov_cap_warn, prov_cap_crit, log_del_interval, metrics_retention_period,
+                   contact_point, grafana_endpoint, distr_bs, distr_chunk_bs,
+                   enable_node_affinity, qpair_count, max_queue_size, inflight_io_threshold, strict_node_anti_affinity,
+                   data_nics,spdk_image,spdk_debug,small_bufsize,large_bufsize,num_partitions_per_dev,jm_percent,
+                   spdk_cpu_mask,max_lvol,max_snap,max_prov,number_of_devices,enable_test_device,enable_ha_jm,
+                   ha_jm_count, number_of_distribs,namespace,secondary_nodes,partition_size,
+                   lvol_name, lvol_size, lvol_ha_type, pool_name, pool_max, host_id, comp, crypto, distr_vuid, max_rw_iops,
+                   max_rw_mbytes, max_r_mbytes, max_w_mbytes, with_snapshot, max_size, crypto_key1, crypto_key2,
+                   lvol_priority_class, id_device_by_nqn, fstype):
+    logger.info("run deploy-cleaner")
+    
+    storage_node_ops.deploy_cleaner()
+    
+    logger.info("creating cluster")
+    cluster_uuid = create_cluster(
+            blk_size, page_size_in_blocks,
+            cli_pass, cap_warn, cap_crit, prov_cap_warn, prov_cap_crit,
+            ifname, log_del_interval, metrics_retention_period, contact_point, grafana_endpoint,
+            distr_ndcs, distr_npcs, distr_bs, distr_chunk_bs, ha_type, enable_node_affinity,
+            qpair_count, max_queue_size, inflight_io_threshold, enable_qos, strict_node_anti_affinity)
+    
+    time.sleep(5)
+
+    storage_nodes_list=storage_nodes.split(",")
+    for node in storage_nodes_list:
+        node_ip = node.strip()
+        dev_ip=f"{node_ip}:5000"
+        add_node_status=storage_node_ops.add_node(cluster_uuid,dev_ip,ifname,data_nics,max_lvol,max_snap,max_prov,spdk_image,spdk_debug,
+                                  small_bufsize,large_bufsize,spdk_cpu_mask,num_partitions_per_dev,jm_percent,number_of_devices,
+                                  enable_test_device,namespace,number_of_distribs,enable_ha_jm,False,id_device_by_nqn,partition_size,ha_jm_count)
+        
+        
+        if not add_node_status:
+            logger.error("Could not add storage node successfully")
+            return False
+
+        time.sleep(5)
+    
+    if secondary_nodes:
+        secondary_nodes_list = secondary_nodes.split(",")
+        for node in secondary_nodes_list:
+            node_ip = node.strip()
+            dev_ip=f"{node_ip}:5000"
+            add_node_status=storage_node_ops.add_node(cluster_uuid,dev_ip,ifname,data_nics,max_lvol,max_snap,max_prov,spdk_image,spdk_debug,
+                                    small_bufsize,large_bufsize,spdk_cpu_mask,num_partitions_per_dev,jm_percent,number_of_devices,
+                                    enable_test_device,namespace,number_of_distribs,enable_ha_jm,True,id_device_by_nqn,partition_size,ha_jm_count)
+                    
+            if not add_node_status:
+                logger.error("Could not add storage node successfully")
+                return False
+
+        time.sleep(5)
+    
+    activated = cluster_activate(cluster_uuid)
+    if not activated:
+        logger.error("cluster failed to activate")
+    
+    if test:
+        if not pool_name:
+            logger.error("Pool name is empty!")
+            return False        
+        pool_id=pool_controller.add_pool(pool_name,pool_max,max_size,
+                            max_rw_iops,max_rw_mbytes,max_r_mbytes,max_w_mbytes,None,cluster_uuid)
+        
+        if not pool_id:
+            logger.error("pool did not create successfully")
+            return False 
+
+        if not lvol_name or lvol_size:
+            logger.error("lvol name or size is empty!")
+            return False   
+            
+        lvol_uuid, msg = lvol_controller.add_lvol_ha(
+                    lvol_name, lvol_size, host_id, lvol_ha_type, pool_id, comp, crypto,
+                    distr_vuid,
+                    max_rw_iops=max_rw_iops,
+                    max_rw_mbytes=max_rw_mbytes,
+                    max_r_mbytes=max_r_mbytes,
+                    max_w_mbytes=max_w_mbytes,
+                    with_snapshot=with_snapshot,
+                    max_size=max_size,
+                    crypto_key1=crypto_key1,
+                    crypto_key2=crypto_key2,
+                    lvol_priority_class=lvol_priority_class,
+                    uid=None, pvc_name=None, namespace=None)
+
+        if not lvol_uuid:
+            logger.error(f"lvol creation failed {msg}")
+        
+        
+        time.sleep(5)
+        
+        subprocess.run("sudo modprobe nvme-tcp", shell=True, check=True)
+
+        connect=lvol_controller.connect_lvol(lvol_uuid)
+        
+        if not connect:
+            logger.error("connect command generation failed")
+
+        for entry in connect:
+            connect_string = entry.get("connect")
+                
+            subprocess.run(connect_string, shell=True, check=True)
+        
+        nvme_list_command = "sudo nvme list"
+        logger.info(f"Executing NVMe list command: {nvme_list_command}")
+        result = subprocess.run(nvme_list_command, shell=True, check=True, capture_output=True, text=True)
+
+        nvme_output = result.stdout
+        device_name = parse_nvme_list_output(nvme_output, lvol_uuid)
+        
+        mkfs_command = f"sudo mkfs.{fstype} {device_name}"
+        subprocess.run(mkfs_command, shell=True, check=True)
+        
+        
+        mount_point = os.path.join(os.path.expanduser("~"), lvol_uuid)
+        os.makedirs(mount_point, exist_ok=True)
+        mount_command = f"sudo mount {device_name} {mount_point}"
+        
+        subprocess.run(mount_command, shell=True, check=True)
+
+        logger.info(f"running fio on mount point {mount_point}")
+
+        run_fio(mount_point)
+        
+        match = re.search(r'--nqn=([^\s]+)', connect_string)
+        nqn_value = match.group(1)
+        
+        cleaned = cleanup_nvme(mount_point, nqn_value)
+        if not cleaned:
+            return False
+
+        status=lvol_controller.delete_lvol(lvol_uuid)
+        
+        pool_controller.delete_pool(pool_id)
+        return True
+    else:
+        return True
+                    
 def add_cluster(blk_size, page_size_in_blocks, cap_warn, cap_crit, prov_cap_warn, prov_cap_crit,
                 distr_ndcs, distr_npcs, distr_bs, distr_chunk_bs, ha_type, enable_node_affinity, qpair_count,
                 max_queue_size, inflight_io_threshold, enable_qos, strict_node_anti_affinity):
@@ -321,11 +537,11 @@ def add_cluster(blk_size, page_size_in_blocks, cap_warn, cap_crit, prov_cap_warn
     _create_update_user(cluster.uuid, cluster.grafana_endpoint, cluster.grafana_secret, cluster.secret)
 
     if distr_ndcs == 0 and distr_npcs == 0:
-        cluster.distr_ndcs = 2
-        cluster.distr_npcs = 1
-    else:
-        cluster.distr_ndcs = distr_ndcs
-        cluster.distr_npcs = distr_npcs
+        logger.error("both distr_ndcs and distr_npcs cannot be 0")
+        return False
+
+    cluster.distr_ndcs = distr_ndcs
+    cluster.distr_npcs = distr_npcs
     cluster.distr_bs = distr_bs
     cluster.distr_chunk_bs = distr_chunk_bs
     cluster.ha_type = ha_type
@@ -364,18 +580,22 @@ def cluster_activate(cl_id, force=False, force_lvstore_create=False):
             return False
 
     ols_status = cluster.status
-    set_cluster_status(cl_id, Cluster.STATUS_IN_ACTIVATION)
+    if ols_status == Cluster.STATUS_IN_ACTIVATION:
+        ols_status = Cluster.STATUS_UNREADY
+    else:
+        set_cluster_status(cl_id, Cluster.STATUS_IN_ACTIVATION)
     snodes = db_controller.get_storage_nodes_by_cluster_id(cl_id)
     online_nodes = []
     dev_count = 0
 
     for node in snodes:
-        if node.is_secondary_node:
+        if node.is_secondary_node:  # pass
             continue
         if node.status == node.STATUS_ONLINE:
             online_nodes.append(node)
             for dev in node.nvme_devices:
-                if dev.status == dev.STATUS_ONLINE:
+                if dev.status in [NVMeDevice.STATUS_ONLINE, NVMeDevice.STATUS_READONLY,
+                                  NVMeDevice.STATUS_CANNOT_ALLOCATE]:
                     dev_count += 1
     minimum_devices = cluster.distr_ndcs + cluster.distr_npcs + 1
     if dev_count < minimum_devices:
@@ -386,20 +606,34 @@ def cluster_activate(cl_id, force=False, force_lvstore_create=False):
     records = db_controller.get_cluster_capacity(cluster)
     max_size = records[0]['size_total']
 
+    used_nodes_as_sec = []
+    snodes = db_controller.get_storage_nodes_by_cluster_id(cl_id)
     if cluster.ha_type == "ha":
         for snode in snodes:
-            if snode.is_secondary_node or snode.secondary_node_id:
+            if snode.is_secondary_node:  # pass
+                continue
+            if snode.secondary_node_id:
+                sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
+                sec_node.lvstore_stack_secondary_1 = snode.get_id()
+                sec_node.write_to_db()
+                used_nodes_as_sec.append(snode.secondary_node_id)
                 continue
             secondary_nodes = storage_node_ops.get_secondary_nodes(snode)
             if not secondary_nodes:
                 logger.error(f"Failed to activate cluster, No enough secondary nodes")
                 set_cluster_status(cl_id, ols_status)
                 return False
+            snode = db_controller.get_storage_node_by_id(snode.get_id())
             snode.secondary_node_id = secondary_nodes[0]
             snode.write_to_db()
+            sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
+            sec_node.lvstore_stack_secondary_1 = snode.get_id()
+            sec_node.write_to_db()
+            used_nodes_as_sec.append(snode.secondary_node_id)
 
+    snodes = db_controller.get_storage_nodes_by_cluster_id(cl_id)
     for snode in snodes:
-        if snode.is_secondary_node:
+        if snode.is_secondary_node:  # pass
             continue
         if snode.status != StorageNode.STATUS_ONLINE:
             continue
@@ -408,23 +642,21 @@ def cluster_activate(cl_id, force=False, force_lvstore_create=False):
             ret = storage_node_ops.recreate_lvstore(snode)
         else:
             ret = storage_node_ops.create_lvstore(snode, cluster.distr_ndcs, cluster.distr_npcs, cluster.distr_bs,
-                                              cluster.distr_chunk_bs, cluster.page_size_in_blocks, max_size, snodes)
-        if not ret and not force:
-            logger.error("Failed to activate cluster")
-            set_cluster_status(cl_id, ols_status)
-            return False
+                                              cluster.distr_chunk_bs, cluster.page_size_in_blocks, max_size)
+        snode = db_controller.get_storage_node_by_id(snode.get_id())
+        if ret:
+            snode.lvstore_status = "ready"
+            snode.write_to_db()
 
-    for snode in snodes:
-        if not snode.is_secondary_node:
-            continue
-        if snode.status != StorageNode.STATUS_ONLINE:
-            continue
+        else:
+            snode.lvstore_status = "failed"
+            snode.write_to_db()
+            logger.error(f"Failed to restore lvstore on node {snode.get_id()}")
+            if not force:
+                logger.error("Failed to activate cluster")
+                set_cluster_status(cl_id, ols_status)
+                return False
 
-        ret = storage_node_ops.recreate_lvstore(snode)
-        if not ret and not force:
-            logger.error("Failed to activate cluster")
-            set_cluster_status(cl_id, ols_status)
-            return False
 
     if not cluster.cluster_max_size:
         cluster = db_controller.get_cluster_by_id(cl_id)
@@ -451,6 +683,7 @@ def get_cluster_status(cl_id, is_json=False):
             data.append({
                 "UUID": dev.get_id(),
                 "Storage ID": dev.cluster_device_order,
+                "Physical label": dev.physical_label,
                 "Size": utils.humanbytes(dev.size),
                 "Hostname": node.hostname,
                 "Status": dev.status,
@@ -507,13 +740,13 @@ def cluster_set_read_only(cl_id):
     if ret:
         st = db_controller.get_storage_nodes_by_cluster_id(cl_id)
         for node in st:
-            rpc_client = RPCClient(
-                node.mgmt_ip, node.rpc_port,
-                node.rpc_username, node.rpc_password, timeout=5, retry=2)
-
-            for bdev in node.lvstore_stack:
-                if bdev['type'] == "bdev_distr":
-                    rpc_client.bdev_distrib_toggle_cluster_full(bdev['name'], cluster_full=True)
+            if node.status not in [StorageNode.STATUS_ONLINE, StorageNode.STATUS_SUSPENDED, StorageNode.STATUS_DOWN]:
+                continue
+            for dev in node.nvme_devices:
+                if dev.status == NVMeDevice.STATUS_ONLINE:
+                    # dev_stat = db_controller.get_device_stats(dev, 1)
+                    # if dev_stat and dev_stat[0].size_util >= cluster.cap_crit:
+                    device_controller.device_set_state(dev.get_id(), NVMeDevice.STATUS_CANNOT_ALLOCATE)
 
     return True
 
@@ -532,13 +765,14 @@ def cluster_set_active(cl_id):
     if ret:
         st = db_controller.get_storage_nodes_by_cluster_id(cl_id)
         for node in st:
-            rpc_client = RPCClient(
-                node.mgmt_ip, node.rpc_port,
-                node.rpc_username, node.rpc_password, timeout=5, retry=2)
+            if node.status not in [StorageNode.STATUS_ONLINE, StorageNode.STATUS_SUSPENDED, StorageNode.STATUS_DOWN]:
+                continue
 
-            for bdev in node.lvstore_stack:
-                if bdev['type'] == "bdev_distr":
-                    rpc_client.bdev_distrib_toggle_cluster_full(bdev['name'], cluster_full=False)
+            for dev in node.nvme_devices:
+                if dev.status in [NVMeDevice.STATUS_CANNOT_ALLOCATE, NVMeDevice.STATUS_READONLY]:
+                    dev_stat = db_controller.get_device_stats(dev, 1)
+                    if dev_stat and dev_stat[0].size_util < cluster.cap_crit:
+                        device_controller.device_set_online(dev.get_id())
 
     return True
 
@@ -551,15 +785,17 @@ def list():
     data = []
     for cl in cls:
         st = db_controller.get_storage_nodes_by_cluster_id(cl.get_id())
+        status = cl.status
+        if cl.is_re_balancing and status in [Cluster.STATUS_ACTIVE, Cluster.STATUS_DEGRADED]:
+            status = f"{status} - ReBalancing"
         data.append({
             "UUID": cl.get_id(),
             "NQN": cl.nqn,
             "ha_type": cl.ha_type,
-            "tls": cl.tls,
-            "mgmt nodes": len(mt),
-            "storage nodes": len(st),
+            "#mgmt": len(mt),
+            "#storage": len(st),
             "Mod": f"{cl.distr_ndcs}x{cl.distr_npcs}",
-            "Status": cl.status,
+            "Status": status.upper(),
         })
     return utils.print_table(data)
 
@@ -612,7 +848,9 @@ def list_all_info(cluster_id):
         elif task.status in [JobSchedule.STATUS_NEW, JobSchedule.STATUS_SUSPENDED]:
             task_pending += 1
 
-
+    status = cl.status
+    if cl.is_re_balancing and status in [Cluster.STATUS_ACTIVE, Cluster.STATUS_DEGRADED]:
+        status = f"{status} - ReBalancing"
     data.append({
         "Cluster UUID": cl.get_id(),
         "Type": cl.ha_type.upper(),
@@ -634,8 +872,7 @@ def list_all_info(cluster_id):
         # "Size prov": f"{utils.humanbytes(rec.size_prov)}",
         # "Size util": f"{rec.size_util}%",
         # "Size prov util": f"{rec.size_prov_util}%",
-
-        "Status": cl.status,
+        "Status": status.upper(),
 
     })
 
@@ -662,7 +899,7 @@ def list_all_info(cluster_id):
         "Write IOP/s": f"{rec.write_io_ps}",
 
         "Health": "True",
-        "Status": cl.status,
+        "Status": status.upper(),
 
     })
 
@@ -690,20 +927,22 @@ def list_all_info(cluster_id):
 
         data.append({
             "Storage node UUID": node.uuid,
-            "Devices": f"{total_devices}/{online_devices}",
-            "LVols": f"{len(lvs)}",
 
-            "Size prov": f"{utils.humanbytes(rec.size_total)}",
-            "Size Used": f"{utils.humanbytes(rec.size_used)}",
-            "Size free": f"{utils.humanbytes(rec.size_free)}",
-            "Size %": f"{rec.size_util}%",
-            "Size prov %": f"{rec.size_prov_util}%",
+            "Size": f"{utils.humanbytes(rec.size_total)}",
+            "Used": f"{utils.humanbytes(rec.size_used)}",
+            "Free": f"{utils.humanbytes(rec.size_free)}",
+            "Util": f"{rec.size_util}%",
 
             "Read BW/s": f"{utils.humanbytes(rec.read_bytes_ps)}",
             "Write BW/s": f"{utils.humanbytes(rec.write_bytes_ps)}",
             "Read IOP/s": f"{rec.read_io_ps}",
             "Write IOP/s": f"{rec.write_io_ps}",
 
+            "Size prov": f"{utils.humanbytes(rec.size_prov)}",
+            "Util prov": f"{rec.size_prov_util}%",
+
+            "Devices": f"{total_devices}/{online_devices}",
+            "LVols": f"{len(lvs)}",
             "Status": node.status,
 
         })
@@ -717,18 +956,15 @@ def list_all_info(cluster_id):
 
             dev_data.append({
                 "Device UUID": dev.uuid,
-
-                "Size total": f"{utils.humanbytes(rec.size_total)}",
-                "Size Used": f"{utils.humanbytes(rec.size_used)}",
-                "Size free": f"{utils.humanbytes(rec.size_free)}",
-                "Size %": f"{rec.size_util}%",
-                # "Size prov %": f"{rec.size_prov_util}%",
-
+                "Size": f"{utils.humanbytes(rec.size_total)}",
+                "Used": f"{utils.humanbytes(rec.size_used)}",
+                "Free": f"{utils.humanbytes(rec.size_free)}",
+                "Util": f"{rec.size_util}%",
                 "Read BW/s": f"{utils.humanbytes(rec.read_bytes_ps)}",
                 "Write BW/s": f"{utils.humanbytes(rec.write_bytes_ps)}",
                 "Read IOP/s": f"{rec.read_io_ps}",
                 "Write IOP/s": f"{rec.write_io_ps}",
-
+                "StorgeID": dev.cluster_device_order,
                 "Health": dev.health_check,
                 "Status": dev.status,
 
@@ -754,18 +990,14 @@ def list_all_info(cluster_id):
 
         lvol_data.append({
             "LVol UUID": lvol.uuid,
-
-            "Size prov": f"{utils.humanbytes(rec.size_total)}",
-            "Size Used": f"{utils.humanbytes(rec.size_used)}",
-            "Size free": f"{utils.humanbytes(rec.size_free)}",
-            "Size %": f"{rec.size_util}%",
-            # "Size prov %": f"{rec.size_prov_util}%",
-
+            "Size": f"{utils.humanbytes(rec.size_total)}",
+            "Used": f"{utils.humanbytes(rec.size_used)}",
+            "Free": f"{utils.humanbytes(rec.size_free)}",
+            "Util": f"{rec.size_util}%",
             "Read BW/s": f"{utils.humanbytes(rec.read_bytes_ps)}",
             "Write BW/s": f"{utils.humanbytes(rec.write_bytes_ps)}",
             "Read IOP/s": f"{rec.read_io_ps}",
             "Write IOP/s": f"{rec.write_io_ps}",
-
             "Connections": f"{rec.connected_clients}",
             "Health": lvol.health_check,
             "Status": lvol.status,
@@ -829,12 +1061,12 @@ def get_iostats_history(cluster_id, history_string, records_count=20, parse_size
     db_controller = DBController()
     cluster = db_controller.get_cluster_by_id(cluster_id)
     if not cluster:
-        logger.error(f"Cluster not found {cluster_id}")
+        logger.warning(f"Cluster not found {cluster_id}")
         return False
 
     nodes = db_controller.get_storage_nodes_by_cluster_id(cluster_id)
     if not nodes:
-        logger.error("no nodes found")
+        logger.warning("no nodes found")
         return False
 
     if history_string:
@@ -957,6 +1189,9 @@ def get_logs(cluster_id, is_json=False):
         if 'storage_ID' in record.object_dict:
             Storage_ID = record.object_dict['storage_ID']
 
+        elif 'cluster_device_order' in record.object_dict:
+            Storage_ID = record.object_dict['cluster_device_order']
+
         vuid = None
         if 'vuid' in record.object_dict:
             vuid = record.object_dict['vuid']
@@ -991,19 +1226,20 @@ def get_cluster(cl_id):
     return json.dumps(cluster.get_clean_dict(), indent=2, sort_keys=True)
 
 
-def update_cluster(cl_id):
+def update_cluster(cl_id, mgmt_only=False, restart_cluster=False):
     db_controller = DBController()
     cluster = db_controller.get_cluster_by_id(cl_id)
     if not cluster:
         logger.error(f"Cluster not found {cl_id}")
         return False
 
-    # try:
-    #     out, _, ret_code = shell_utils.run_command("pip install sbcli-dev --upgrade")
-    #     if ret_code == 0:
-    #         logger.info("sbcli-dev is upgraded")
-    # except Exception as e:
-    #     logger.error(e)
+    try:
+        sbcli=constants.SIMPLY_BLOCK_CLI_NAME
+        out, _, ret_code = shell_utils.run_command(f"pip install {sbcli} --upgrade")
+        if ret_code == 0:
+            logger.info(f"{sbcli} upgraded")
+    except Exception as e:
+        logger.error(e)
 
     try:
         logger.info("Updating mgmt cluster")
@@ -1011,19 +1247,30 @@ def update_cluster(cl_id):
         logger.info(f"Pulling image {constants.SIMPLY_BLOCK_DOCKER_IMAGE}")
         cluster_docker.images.pull(constants.SIMPLY_BLOCK_DOCKER_IMAGE)
         image_without_tag = constants.SIMPLY_BLOCK_DOCKER_IMAGE.split(":")[0]
+        image_without_tag = image_without_tag.split("/")[-1]
         for service in cluster_docker.services.list():
             if image_without_tag in service.attrs['Spec']['Labels']['com.docker.stack.image']:
                 logger.info(f"Updating service {service.name}")
                 service.update(image=constants.SIMPLY_BLOCK_DOCKER_IMAGE, force_update=True)
-        logger.info("Done")
+        logger.info("Done updating mgmt cluster")
     except Exception as e:
         print(e)
 
-    for node in db_controller.get_storage_nodes_by_cluster_id(cl_id):
-        try:
-            storage_node_ops.start_storage_node_api_container(node.mgmt_ip)
-        except:
-            pass
+    if not mgmt_only:
+        set_cluster_status(cl_id, Cluster.STATUS_SUSPENDED)
+        logger.info("updating storage nodes")
+        for node in db_controller.get_storage_nodes_by_cluster_id(cl_id):
+            if node.status in [StorageNode.STATUS_ONLINE, StorageNode.STATUS_SUSPENDED]:
+                try:
+                    storage_node_ops.start_storage_node_api_container(node.mgmt_ip)
+                except Exception as e:
+                    logger.error(e)
+
+        if restart_cluster:
+            logger.info("Restarting cluster")
+            ret = cluster_grace_shutdown(cl_id)
+            if ret:
+                cluster_grace_startup(cl_id)
 
     logger.info("Done")
     return True
@@ -1059,13 +1306,12 @@ def cluster_grace_shutdown(cl_id):
 
     st = db_controller.get_storage_nodes_by_cluster_id(cl_id)
     for node in st:
-        logger.info(f"Suspending node: {node.get_id()}")
-        storage_node_ops.suspend_storage_node(node.get_id())
-        logger.info(f"Shutting down node: {node.get_id()}")
-        storage_node_ops.shutdown_storage_node(node.get_id())
+        if node.status == StorageNode.STATUS_ONLINE:
+            logger.info(f"Suspending node: {node.get_id()}")
+            storage_node_ops.suspend_storage_node(node.get_id())
+            logger.info(f"Shutting down node: {node.get_id()}")
+            storage_node_ops.shutdown_storage_node(node.get_id())
 
-    logger.info(f"Suspending cluster: {cl_id}")
-    suspend_cluster(cl_id)
     return True
 
 
@@ -1136,48 +1382,20 @@ def open_db_from_zip(fip_path):
 
 
 
-def cluster_reset():
-    """
+def set(cl_id, attr, value):
+    db_controller = DBController()
+    cluster = db_controller.get_cluster_by_id(cl_id)
+    if not cluster:
+        logger.error(f"Cluster not found {cl_id}")
+        return False
 
+    if attr in cluster.get_attrs_map():
+        try:
+            value = cluster.get_attrs_map()[attr]['type'](value)
+            logger.info(f"Setting {attr} to {value}")
+            setattr(cluster, attr, value)
+            cluster.write_to_db()
+        except:
+            pass
 
-set -x
-
-CMD=$(ls ~/.local/bin/sbcli-* | awk '{n=split($0,a,"/"); print a[n]}')
-cl=$($CMD cluster list | tail -n -3 | awk '{print $2}')
-
-#$CMD cluster graceful-shutdown $cl
-
-for sn_id in $($CMD sn list | grep / | awk '{print $2}'); do
-  $CMD -d sn shutdown --force $sn_id
-done
-
-sudo mv /etc/foundationdb/fdb.cluster /etc/foundationdb/fdb.cluster.bck
-for service_id in $(docker service ls | grep / | awk '{print $1}'); do
-  docker service update "$service_id" --force --detach
-done
-
-# restore
-fdb_cont=$(sudo docker ps | grep "app_fdb-server" | awk '{print $1}')
-sudo docker rm --force $fdb_cont
-sudo rm -rf /etc/foundationdb/data/*
-fdbcli --exec "configure new single ssd ; writemode on ; clearrange \"\" \\xff" -C /etc/foundationdb/fdb.cluster.bck
-BF=$(fdbbackup list -b file:///etc/foundationdb/backup/)
-fdbrestore start -r "$BF" --dest-cluster-file /etc/foundationdb/fdb.cluster.bck -t fresh_deploy
-
-sudo mv /etc/foundationdb/fdb.cluster.bck /etc/foundationdb/fdb.cluster
-
-for service_id in $(docker service ls | grep / | awk '{print $1}'); do
-  docker service update "$service_id" --force --detach
-done
-
-sleep 30
-for sn_id in $($CMD sn list | grep / | awk '{print $2}'); do
-  $CMD -d sn shutdown --force $sn_id
-done
-
-sleep 5
-$CMD -d cluster graceful-startup $cl  --clear-data
-
-$CMD -d cluster activate $cl
-
-    """
\ No newline at end of file
+    return True
