@@ -15,8 +15,6 @@ from simplyblock_core.models.pool import Pool
 
 logger = lg.getLogger()
 
-db_controller = DBController()
-
 
 def _generate_string(length):
     return ''.join(random.SystemRandom().choice(
@@ -24,7 +22,7 @@ def _generate_string(length):
 
 
 def add_pool(name, pool_max, lvol_max, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes, has_secret, cluster_id):
-
+    db_controller = DBController()
     if not name:
         logger.error("Pool name is empty!")
         return False
@@ -85,6 +83,7 @@ def set_pool_value_if_above(pool, key, value):
 
 def set_pool(uuid, pool_max, lvol_max, max_rw_iops,
              max_rw_mbytes, max_r_mbytes, max_w_mbytes):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(uuid)
     if not pool:
         logger.error(f"Pool not found {uuid}")
@@ -121,7 +120,10 @@ def set_pool(uuid, pool_max, lvol_max, max_rw_iops,
 
 
 def delete_pool(uuid):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(uuid)
+    if not pool:
+        pool = db_controller.get_pool_by_name(uuid)
     if not pool:
         logger.error(f"Pool not found {uuid}")
         return False
@@ -129,9 +131,10 @@ def delete_pool(uuid):
         logger.error(f"Pool is disabled")
         return False
 
-    # if pool.lvols:
-    #     logger.error(f"Pool is not empty {uuid}")
-    #     return False
+    lvols = db_controller.get_lvols_by_pool_id(uuid)
+    if lvols and len(lvols) > 0:
+        logger.error(f"Pool {uuid} is not empty, lvols found {len(lvols)}")
+        return False
 
     logger.info(f"Deleting pool {pool.get_id()}")
     pool_events.pool_remove(pool)
@@ -141,6 +144,7 @@ def delete_pool(uuid):
 
 
 def list_pools(is_json, cluster_id=None):
+    db_controller = DBController()
     pools = db_controller.get_pools(cluster_id)
     data = []
     for pool in pools:
@@ -163,6 +167,7 @@ def list_pools(is_json, cluster_id=None):
 
 
 def set_status(pool_id, status):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     logger.info(f"Setting pool:{pool_id} status to Active")
     if not pool:
@@ -174,6 +179,7 @@ def set_status(pool_id, status):
 
 
 def get_pool(pool_id, is_json):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -188,6 +194,7 @@ def get_pool(pool_id, is_json):
 
 
 def get_capacity(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -214,6 +221,7 @@ def get_capacity(pool_id):
 
 
 def get_io_stats(pool_id, history, records_count=20):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -245,6 +253,7 @@ def get_io_stats(pool_id, history, records_count=20):
 
 
 def get_secret(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -257,6 +266,7 @@ def get_secret(pool_id):
 
 
 def set_secret(pool_id, secret):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -271,6 +281,7 @@ def set_secret(pool_id, secret):
 
 
 def get_pool_total_capacity(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -287,6 +298,7 @@ def get_pool_total_capacity(pool_id):
 
 
 def get_pool_total_rw_iops(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -302,6 +314,7 @@ def get_pool_total_rw_iops(pool_id):
 
 
 def get_pool_total_rw_mbytes(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -317,6 +330,7 @@ def get_pool_total_rw_mbytes(pool_id):
 
 
 def get_pool_total_r_mbytes(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
@@ -332,6 +346,7 @@ def get_pool_total_r_mbytes(pool_id):
 
 
 def get_pool_total_w_mbytes(pool_id):
+    db_controller = DBController()
     pool = db_controller.get_pool_by_id(pool_id)
     if not pool:
         logger.error(f"Pool not found {pool_id}")
