@@ -25,13 +25,15 @@ db_controller = DBController()
 def add(lvol_id, snapshot_name):
     lvol = db_controller.get_lvol_by_id(lvol_id)
     if not lvol:
-        logger.error(f"LVol not found: {lvol_id}")
-        return False
+        msg = f"LVol not found: {lvol_id}"
+        logger.error(msg)
+        return False, msg
 
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
-        return False
+        msg = f"Pool is disabled"
+        logger.error(msg)
+        return False, msg
 
     if lvol.cloned_from_snap:
         snap = db_controller.get_snapshot_by_id(lvol.cloned_from_snap)
@@ -53,12 +55,6 @@ def add(lvol_id, snapshot_name):
     logger.info(f"Creating snapshot: {snapshot_name} from LVol: {lvol.get_id()}")
     snode = db_controller.get_storage_node_by_id(lvol.node_id)
 
-    if snode.status != StorageNode.STATUS_ONLINE:
-        msg = f"Node is not online, {snode.status}"
-        logger.error(msg)
-        return False, msg
-
-
     rec = db_controller.get_lvol_stats(lvol, 1)
     if rec:
         size = rec[0].size_used
@@ -66,37 +62,16 @@ def add(lvol_id, snapshot_name):
         size = lvol.size
 
     if 0 < pool.lvol_max_size < size:
-        logger.error(f"Pool Max LVol size is: {utils.humanbytes(pool.lvol_max_size)}, LVol size: {utils.humanbytes(size)} must be below this limit")
-        return False
+        msg = f"Pool Max LVol size is: {utils.humanbytes(pool.lvol_max_size)}, LVol size: {utils.humanbytes(size)} must be below this limit"
+        logger.error(msg)
+        return False, msg
 
     if pool.pool_max_size > 0:
         total = pool_controller.get_pool_total_capacity(pool.get_id())
         if total + size > pool.pool_max_size:
-            logger.error( f"Invalid LVol size: {utils.humanbytes(size)} " 
-                          f"Pool max size has reached {utils.humanbytes(total+size)} of {utils.humanbytes(pool.pool_max_size)}")
-            return False
-
-
-##############################################################################
-    # # Validate adding snap on storage node
-    # snode_api = SNodeClient(snode.api_endpoint)
-    # result, _ = snode_api.info()
-    # memory_free = result["memory_details"]["free"]
-    # huge_free = result["memory_details"]["huge_free"]
-    # total_node_capacity = db_controller.get_snode_size(snode.get_id())
-    #
-    # error = utils.validate_add_lvol_or_snap_on_node(
-    #     memory_free,
-    #     huge_free,
-    #     snode.max_snap,
-    #     lvol.size,
-    #     total_node_capacity,
-    #     len(db_controller.get_snapshots_by_node_id(lvol.node_id)))
-    #
-    # if error:
-    #     logger.error(f"Failed to add snap on node {lvol.node_id}")
-    #     logger.error(error)
-    #     return False
+            msg =  f"Invalid LVol size: {utils.humanbytes(size)}. pool max size has reached {utils.humanbytes(total+size)} of {utils.humanbytes(pool.pool_max_size)}"
+            logger.error(msg)
+            return False, msg
 
     if pool.pool_max_size > 0:
         total = pool_controller.get_pool_total_capacity(pool.get_id())
@@ -105,54 +80,103 @@ def add(lvol_id, snapshot_name):
             logger.error(msg)
             return False, msg
 
-    # if snode.max_snap:
-    #     cnt = db_controller.get_snapshots_by_node_id(snode.get_id())
-    #     if cnt and len(cnt)+1 > snode.max_snap:
-    #         msg = f"Storage node snapshots count must be less than max_snap:{snode.max_snap}"
-    #         logger.error(msg)
-    #         return False, msg
-
-##############################################################################
+    cluster = db_controller.get_cluster_by_id(pool.cluster_id)
+    if cluster.status not in [cluster.STATUS_ACTIVE, cluster.STATUS_DEGRADED]:
+        return False, f"Cluster is not active, status: {cluster.status}"
 
     snap_bdev_name = f"SNAP_{utils.get_random_vuid()}"
-    rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
-
+    size = lvol.size
     blobid = 0
     snap_uuid = ""
-    rpc_client.bdev_lvol_set_leader(True, lvs_name=lvol.lvs_name)
-    logger.info("Creating Snapshot bdev")
-    ret = rpc_client.lvol_create_snapshot(f"{lvol.lvs_name}/{lvol.lvol_bdev}", snap_bdev_name)
-    if not ret:
-        return False, f"Failed to create snapshot on node: {snode.get_id()}"
-
-    size = lvol.size
     used_size = 0
-    snap_bdev = rpc_client.get_bdevs(f"{lvol.lvs_name}/{snap_bdev_name}")
-    if snap_bdev:
-        snap_uuid = snap_bdev[0]['uuid']
-        blobid = snap_bdev[0]['driver_specific']['lvol']['blobid']
-        cluster = db_controller.get_cluster_by_id(pool.cluster_id)
-        cluster_size = cluster.distr_ndcs * cluster.page_size_in_blocks
-        num_allocated_clusters = snap_bdev[0]["driver_specific"]["lvol"]["num_allocated_clusters"]
-        used_size = int(num_allocated_clusters*cluster_size)
 
-    if snode.secondary_node_id and blobid:
-        sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
-        if sec_node.status == StorageNode.STATUS_ONLINE:
-            sec_rpc_client = RPCClient(sec_node.mgmt_ip, sec_node.rpc_port, sec_node.rpc_username, sec_node.rpc_password)
-            sec_rpc_client.bdev_lvol_set_leader(False, lvs_name=lvol.lvs_name)
+    if lvol.ha_type == "single":
+        if snode.status == StorageNode.STATUS_ONLINE:
+            rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
+            logger.info("Creating Snapshot bdev")
+            ret = rpc_client.lvol_create_snapshot(f"{lvol.lvs_name}/{lvol.lvol_bdev}", snap_bdev_name)
+            if not ret:
+                return False, f"Failed to create snapshot on node: {snode.get_id()}"
+
+            snap_bdev = rpc_client.get_bdevs(f"{lvol.lvs_name}/{snap_bdev_name}")
+            if snap_bdev:
+                snap_uuid = snap_bdev[0]['uuid']
+                blobid = snap_bdev[0]['driver_specific']['lvol']['blobid']
+                cluster_size = cluster.distr_ndcs * cluster.page_size_in_blocks
+                num_allocated_clusters = snap_bdev[0]["driver_specific"]["lvol"]["num_allocated_clusters"]
+                used_size = int(num_allocated_clusters*cluster_size)
+        else:
+            msg = f"Host node is not online {snode.get_id()}"
+            logger.error(msg)
+            return False, msg
+
+    if lvol.ha_type == "ha":
+        primary_node = None
+        secondary_node = None
+        host_node = db_controller.get_storage_node_by_id(snode.get_id())
+        sec_node = db_controller.get_storage_node_by_id(host_node.secondary_node_id)
+        lvol.nodes = [host_node.get_id(), host_node.secondary_node_id]
+
+        if host_node.status == StorageNode.STATUS_ONLINE:
+            primary_node = host_node
+            if sec_node.status == StorageNode.STATUS_ONLINE:
+                secondary_node = sec_node
+            elif sec_node.status == StorageNode.STATUS_DOWN:
+                msg = f"Secondary node is in down status, can not create snapshot"
+                logger.error(msg)
+                return False, msg
+            else:
+                # sec node is not online, set primary as leader
+                secondary_node = None
+
+        elif sec_node.status == StorageNode.STATUS_ONLINE:
+            # create on secondary and set leader if needed,
+            secondary_node = None
+            primary_node = sec_node
+
+        else:
+            # both primary and secondary are not online
+            msg = f"Host nodes are not online"
+            logger.error(msg)
+            return False, msg
+
+        if primary_node:
+            rpc_client = RPCClient(
+                primary_node.mgmt_ip, primary_node.rpc_port, primary_node.rpc_username, primary_node.rpc_password)
+
+            logger.info("Creating Snapshot bdev")
+            ret = rpc_client.lvol_create_snapshot(f"{lvol.lvs_name}/{lvol.lvol_bdev}", snap_bdev_name)
+            if not ret:
+                return False, f"Failed to create snapshot on node: {snode.get_id()}"
+
+            snap_bdev = rpc_client.get_bdevs(f"{lvol.lvs_name}/{snap_bdev_name}")
+            if snap_bdev:
+                snap_uuid = snap_bdev[0]['uuid']
+                blobid = snap_bdev[0]['driver_specific']['lvol']['blobid']
+                cluster_size = cluster.distr_ndcs * cluster.page_size_in_blocks
+                num_allocated_clusters = snap_bdev[0]["driver_specific"]["lvol"]["num_allocated_clusters"]
+                used_size = int(num_allocated_clusters*cluster_size)
+            else:
+                return False, f"Failed to create snapshot on node: {snode.get_id()}"
+
+
+        if secondary_node:
+            sec_rpc_client = RPCClient(
+                secondary_node.mgmt_ip, secondary_node.rpc_port, secondary_node.rpc_username, secondary_node.rpc_password)
+
             ret = sec_rpc_client.bdev_lvol_snapshot_register(
                 f"{lvol.lvs_name}/{lvol.lvol_bdev}", snap_bdev_name, snap_uuid, blobid)
             if not ret:
                 msg = f"Failed to register snapshot on node: {sec_node.get_id()}"
                 logger.error(msg)
-                logger.info(f"Removing snapshot from {snode.get_id()}")
+                logger.info(f"Removing snapshot from {primary_node.get_id()}")
+                rpc_client = RPCClient(
+                    primary_node.mgmt_ip, primary_node.rpc_port, primary_node.rpc_username, primary_node.rpc_password)
                 ret = rpc_client.delete_lvol(f"{lvol.lvs_name}/{snap_bdev_name}")
                 if not ret:
                     logger.error(f"Failed to delete snap from node: {snode.get_id()}")
                 return False, msg
 
-
     snap = SnapShot()
     snap.uuid = str(uuid.uuid4())
     snap.snap_uuid = snap_uuid
@@ -181,7 +205,7 @@ def add(lvol_id, snapshot_name):
 
     logger.info("Done")
     snapshot_events.snapshot_create(snap)
-    return snap.uuid
+    return snap.uuid, False
 
 
 def list(all=False):
@@ -229,45 +253,92 @@ def delete(snapshot_uuid, force_delete=False):
 
     logger.info(f"Removing snapshot: {snapshot_uuid}")
 
-    # creating RPCClient instance
-    rpc_client = RPCClient(
-        snode.mgmt_ip,
-        snode.rpc_port,
-        snode.rpc_username,
-        snode.rpc_password)
+    if snap.lvol.ha_type == "single":
+        if snode.status == StorageNode.STATUS_ONLINE:
+            rpc_client = RPCClient(
+                snode.mgmt_ip,
+                snode.rpc_port,
+                snode.rpc_username,
+                snode.rpc_password)
 
-    ret = rpc_client.bdev_lvol_set_leader(True, lvs_name=snap.lvol.lvs_name)
-    if not ret:
-        logger.error(f"Failed to set leader for primary node: {snode.get_id()}")
-        if not force_delete:
+            ret = rpc_client.delete_lvol(snap.snap_bdev)
+            if not ret:
+                logger.error(f"Failed to delete snap from node: {snode.get_id()}")
+                if not force_delete:
+                    return False
+
+        else:
+            msg = f"Host node is not online {snode.get_id()}"
+            logger.error(msg)
             return False
 
-    ret = rpc_client.delete_lvol(snap.snap_bdev)
-    if not ret:
-        logger.error(f"Failed to delete snap from node: {snode.get_id()}")
-        if not force_delete:
-            return False
+    else:
 
-    if snode.secondary_node_id:
+        secondary_node = None
+        primary_node = None
+        host_node = db_controller.get_storage_node_by_id(snode.get_id())
         sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
-        if sec_node.status == StorageNode.STATUS_ONLINE:
-            sec_rpc_client = RPCClient(
-                sec_node.mgmt_ip,
-                sec_node.rpc_port,
-                sec_node.rpc_username,
-                sec_node.rpc_password)
+        if host_node.status == StorageNode.STATUS_ONLINE:
+            if lvol_controller.is_node_leader(host_node, snap.lvol.lvs_name):
+                primary_node = host_node
+                if sec_node.status == StorageNode.STATUS_DOWN:
+                    msg = f"Secondary node is in down status, can not delete snapshot"
+                    logger.error(msg)
+                    return False
 
-            ret = sec_rpc_client.bdev_lvol_set_leader(False, lvs_name=snap.lvol.lvs_name)
+                elif sec_node.status == StorageNode.STATUS_ONLINE:
+                    secondary_node = sec_node
+                else:
+                    secondary_node = None
+
+            elif sec_node.status == StorageNode.STATUS_ONLINE:
+                if lvol_controller.is_node_leader(sec_node, snap.lvol.lvs_name):
+                    primary_node = sec_node
+                    secondary_node = host_node
+                else:
+                    # both nodes are non leaders and online, set primary as leader
+                    primary_node = host_node
+                    secondary_node = sec_node
+
+            else:
+                # sec node is not online, set primary as leader
+                primary_node = host_node
+                secondary_node = None
+
+        elif sec_node.status == StorageNode.STATUS_ONLINE:
+            # primary is not online but secondary is, create on secondary and set leader if needed,
+            secondary_node = None
+            primary_node = sec_node
+
+        else:
+            # both primary and secondary are not online
+            msg = f"Host nodes are not online"
+            logger.error(msg)
+            return False
+
+        if primary_node:
+
+            rpc_client = RPCClient(primary_node.mgmt_ip, primary_node.rpc_port, primary_node.rpc_username,
+                                       primary_node.rpc_password)
+
+            ret = rpc_client.delete_lvol(snap.snap_bdev)
             if not ret:
-                logger.error(f"Failed to set leader for secondary node: {sec_node.get_id()}")
-                if not force_delete:
-                    return False
-            ret = sec_rpc_client.delete_lvol(snap.snap_bdev)
-            if not ret:
-                logger.error(f"Failed to delete snap from node: {sec_node.get_id()}")
+                logger.error(f"Failed to delete snap from node: {snode.get_id()}")
                 if not force_delete:
                     return False
 
+        if secondary_node:
+            secondary_node = db_controller.get_storage_node_by_id(secondary_node.get_id())
+            if secondary_node.status == StorageNode.STATUS_ONLINE:
+                sec_rpc_client = RPCClient(secondary_node.mgmt_ip, secondary_node.rpc_port, secondary_node.rpc_username,
+                                           secondary_node.rpc_password)
+                ret = sec_rpc_client.delete_lvol(snap.snap_bdev)
+                if not ret:
+                    logger.error(f"Failed to delete snap from node: {secondary_node.get_id()}")
+                    if not force_delete:
+                        return False
+
+
     snap.remove(db_controller.kv_store)
 
     base_lvol = db_controller.get_lvol_by_id(snap.lvol.get_id())
@@ -303,10 +374,9 @@ def clone(snapshot_id, clone_name, new_size=0):
         logger.error(msg)
         return False, msg
 
-    if snode.status != snode.STATUS_ONLINE:
-        msg = "Storage node in not Online"
-        logger.error(msg)
-        return False, msg
+    cluster = db_controller.get_cluster_by_id(pool.cluster_id)
+    if cluster.status not in [cluster.STATUS_ACTIVE, cluster.STATUS_DEGRADED]:
+        return False, f"Cluster is not active, status: {cluster.status}"
 
     ref_count = snap.ref_count
     if snap.snap_ref_id:
@@ -327,15 +397,16 @@ def clone(snapshot_id, clone_name, new_size=0):
 
     size = snap.size
     if 0 < pool.lvol_max_size < size:
-        logger.error(f"Pool Max LVol size is: {utils.humanbytes(pool.lvol_max_size)}, LVol size: {utils.humanbytes(size)} must be below this limit")
-        return False
+        msg = f"Pool Max LVol size is: {utils.humanbytes(pool.lvol_max_size)}, LVol size: {utils.humanbytes(size)} must be below this limit"
+        logger.error(msg)
+        return False, msg
 
     if pool.pool_max_size > 0:
         total = pool_controller.get_pool_total_capacity(pool.get_id())
         if total + size > pool.pool_max_size:
-            logger.error( f"Invalid LVol size: {utils.humanbytes(size)} " 
-                          f"Pool max size has reached {utils.humanbytes(total+size)} of {utils.humanbytes(pool.pool_max_size)}")
-            return False
+            msg =  f"Invalid LVol size: {utils.humanbytes(size)}. Pool max size has reached {utils.humanbytes(total+size)} of {utils.humanbytes(pool.pool_max_size)}"
+            logger.error(msg)
+            return False, msg
 
     lvol_count = len(db_controller.get_lvols_by_node_id(snode.get_id()))
     if lvol_count >= snode.max_lvol:
@@ -350,9 +421,6 @@ def clone(snapshot_id, clone_name, new_size=0):
             logger.error(msg)
             return False, msg
 
-
-    cluster = db_controller.get_cluster_by_id(snode.cluster_id)
-
     lvol = LVol()
     lvol.uuid = str(uuid.uuid4())
     lvol.lvol_name = clone_name
@@ -374,6 +442,7 @@ def clone(snapshot_id, clone_name, new_size=0):
     lvol.guid = lvol_controller._generate_hex_string(16)
     lvol.vuid = snap.lvol.vuid
     lvol.snapshot_name = snap.snap_bdev
+    lvol.subsys_port = snap.lvol.subsys_port
 
     lvol.status = LVol.STATUS_ONLINE
     lvol.bdev_stack = [
@@ -418,33 +487,76 @@ def clone(snapshot_id, clone_name, new_size=0):
 
     lvol.write_to_db(db_controller.kv_store)
 
-    lvol_bdev, error = lvol_controller.add_lvol_on_node(lvol, snode)
-    if error:
-        return False, error
-    lvol.nodes = [snode.get_id()]
-    lvol.lvol_uuid = lvol_bdev['uuid']
-    lvol.blobid = lvol_bdev['driver_specific']['lvol']['blobid']
+    if lvol.ha_type == "single":
+        lvol_bdev, error = lvol_controller.add_lvol_on_node(lvol, snode)
+        if error:
+            return False, error
+        lvol.nodes = [snode.get_id()]
+        lvol.lvol_uuid = lvol_bdev['uuid']
+        lvol.blobid = lvol_bdev['driver_specific']['lvol']['blobid']
 
-    if snap.lvol.ha_type == "ha":
-        sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
-        if sec_node.status == StorageNode.STATUS_ONLINE:
-            ret, error = lvol_controller.add_lvol_on_node(lvol, sec_node, ha_inode_self=1)
+    if lvol.ha_type == "ha":
+        host_node = snode
+        lvol.nodes = [host_node.get_id(), host_node.secondary_node_id]
+        primary_node = None
+        secondary_node = None
+        sec_node = db_controller.get_storage_node_by_id(host_node.secondary_node_id)
+        if host_node.status == StorageNode.STATUS_ONLINE:
+
+            if lvol_controller.is_node_leader(host_node, lvol.lvs_name):
+                primary_node = host_node
+                if sec_node.status == StorageNode.STATUS_DOWN:
+                    msg = f"Secondary node is in down status, can not clone snapshot"
+                    logger.error(msg)
+                    lvol.remove(db_controller.kv_store)
+                    return False, msg
+
+                if sec_node.status == StorageNode.STATUS_ONLINE:
+                    secondary_node = sec_node
+
+            elif sec_node.status == StorageNode.STATUS_ONLINE:
+                if lvol_controller.is_node_leader(sec_node, lvol.lvs_name):
+                    primary_node = sec_node
+                    secondary_node = host_node
+                else:
+                    # both nodes are non leaders and online, set primary as leader
+                    primary_node = host_node
+                    secondary_node = sec_node
+
+            else:
+                # sec node is not online, set primary as leader
+                primary_node = host_node
+                secondary_node = None
+
+        elif sec_node.status == StorageNode.STATUS_ONLINE:
+            # create on secondary and set leader if needed,
+            secondary_node = None
+            primary_node = sec_node
+
+        else:
+            # both primary and secondary are not online
+            msg = f"Host nodes are not online"
+            logger.error(msg)
+            lvol.remove(db_controller.kv_store)
+            return False, msg
+
+
+        if primary_node:
+            lvol_bdev, error = lvol_controller.add_lvol_on_node(lvol, primary_node)
             if error:
                 logger.error(error)
-                logger.error(f"Failed to add clone on node: {sec_node.get_id()}")
-                logger.info(f"Removing clone from {snode.get_id()}")
-                lvol.status = LVol.STATUS_IN_DELETION
-                lvol.write_to_db(db_controller.kv_store)
-                ret = lvol_controller.delete_lvol_from_node(lvol.get_id(), snode.get_id())
-                if ret:
-                    lvol.remove(db_controller.kv_store)
-                else:
-                    logger.error(f"Failed to remove clone from node {snode.get_id()}, LVol status: {lvol.status}")
-
+                lvol.remove(db_controller.kv_store)
                 return False, error
 
-        lvol.nodes.append(snode.secondary_node_id)
+            lvol.lvol_uuid = lvol_bdev['uuid']
+            lvol.blobid = lvol_bdev['driver_specific']['lvol']['blobid']
 
+        if secondary_node:
+            lvol_bdev, error = lvol_controller.add_lvol_on_node(lvol, secondary_node, is_primary=False)
+            if error:
+                logger.error(error)
+                lvol.remove(db_controller.kv_store)
+                return False, error
 
     lvol.write_to_db(db_controller.kv_store)
 
@@ -460,4 +572,4 @@ def clone(snapshot_id, clone_name, new_size=0):
     snapshot_events.snapshot_clone(snap, lvol)
     if new_size:
         lvol_controller.resize_lvol(lvol.get_id(), new_size)
-    return True, lvol.uuid
+    return lvol.uuid, False
