@@ -32,7 +32,7 @@ def _validate_new_task_node_restart(cluster_id, node_id):
     tasks = db_controller.get_job_tasks(cluster_id)
     for task in tasks:
         if task.function_name == JobSchedule.FN_NODE_RESTART and task.node_id == node_id:
-            if task.status != JobSchedule.STATUS_DONE:
+            if task.status != JobSchedule.STATUS_DONE and task.canceled is False:
                 return task.get_id()
     return False
 
@@ -81,9 +81,6 @@ def add_device_mig_task(device_id):
     for node in db_controller.get_storage_nodes_by_cluster_id(device.cluster_id):
         if node.status == StorageNode.STATUS_REMOVED:
             continue
-        lvols = db_controller.get_lvols_by_node_id(node.get_id())
-        if not lvols:
-            continue
 
         for bdev in node.lvstore_stack:
             if bdev['type'] == "bdev_distr":
@@ -121,8 +118,12 @@ def list_tasks(cluster_id, is_json=False):
 
         upd = task.updated_at
         if upd:
-            upd = datetime.datetime.strptime(upd, "%Y-%m-%d %H:%M:%S.%f").strftime(
-                "%H:%M:%S, %d/%m/%Y")
+            try:
+                parsed = datetime.datetime.fromisoformat(upd)
+                upd = parsed.strftime("%H:%M:%S, %d/%m/%Y")
+            except Exception as e:
+                logger.error(e)
+
         data.append({
             "Task ID": task.uuid,
             "Node ID / Device ID": f"{task.node_id}\n{task.device_id}",
@@ -130,7 +131,7 @@ def list_tasks(cluster_id, is_json=False):
             "Retry": retry,
             "Status": task.status,
             "Result": task.function_result,
-            "Updated At": upd,
+            "Updated At": upd or "",
         })
     return utils.print_table(data)
 
@@ -206,13 +207,14 @@ def add_node_add_task(cluster_id, function_params):
     return _add_task(JobSchedule.FN_NODE_ADD, cluster_id, "", "", function_params=function_params)
 
 
-def get_active_node_task(cluster_id, node_id):
+def get_active_node_tasks(cluster_id, node_id):
     tasks = db_controller.get_job_tasks(cluster_id)
+    out = []
     for task in tasks:
         if task.node_id == node_id:
             if task.status != JobSchedule.STATUS_DONE and task.canceled is False:
-                return task.uuid
-    return False
+                out.append(task)
+    return out
 
 
 def get_new_device_mig_task(cluster_id, node_id, distr_name, dev_id=None):
