@@ -53,6 +53,7 @@ class StorageNode(BaseNodeObject):
     lvstore_stack: List[dict] = []
     lvstore_stack_secondary_1: List[dict] = []
     lvstore_stack_secondary_2: List[dict] = []
+    lvol_subsys_port: int = 9090
     max_lvol: int = 0
     max_prov: int = 0
     max_snap: int = 0
@@ -65,8 +66,9 @@ class StorageNode(BaseNodeObject):
     number_of_distribs: int = 4
     nvme_devices: List[NVMeDevice] = []
     online_since: str = ""
-    partitions_count: int = 0
+    partitions_count: int = 0  # Unused
     poller_cpu_cores: List[int] = []
+    ssd_pcie: List = []
     pollers_mask: str = ""
     primary_ip: str = ""
     raid: str = ""
@@ -76,8 +78,8 @@ class StorageNode(BaseNodeObject):
     rpc_port: int = -1
     rpc_username: str = ""
     secondary_node_id: str = ""
-    sequential_number: int = 0
-    services: List[str] = []
+    sequential_number: int = 0  # Unused
+    jm_ids: List[str] = []
     spdk_cpu_mask: str = ""
     spdk_debug: bool = False
     spdk_image: str = ""
@@ -85,3 +87,6 @@ class StorageNode(BaseNodeObject):
     partition_size: int = 0
     subsystem: str = ""
     system_uuid: str = ""
+    lvstore_status: str = ""
+    nvmf_port: int = 4420
+    physical_label: int = 0
