@@ -122,7 +122,7 @@ class RPCClient:
             params = {"trtype": trtype}
         return self._request("nvmf_get_transports", params)
 
-    def transport_create(self, trtype, qpair_count=256):
+    def transport_create(self, trtype, qpair_count=6):
         """
             [{'trtype': 'TCP', 'max_queue_depth': 128,
                'max_io_qpairs_per_ctrlr': 127, 'in_capsule_data_size': 4096,
@@ -138,23 +138,29 @@ class RPCClient:
         params = {
             "trtype": trtype,
             "max_io_qpairs_per_ctrlr": qpair_count,
-            "max_queue_depth": 2048,
+            "max_queue_depth": 64,
             "abort_timeout_sec": 5,
             "ack_timeout": 2048,
             "zcopy": True,
-            "in_capsule_data_size": 4096,
+            "in_capsule_data_size": 24500,
             "max_io_size": 131072,
             "io_unit_size": 8192,
             "max_aq_depth": 128,
-            "num_shared_buffers": 8192,
-            "buf_cache_size": 32,
+            "num_shared_buffers": 24576,
+            "buf_cache_size": 512,
             "dif_insert_or_strip": False,
             "c2h_success": True,
             "sock_priority": 0
-
         }
         return self._request("nvmf_create_transport", params)
 
+    def sock_impl_set_options(self):
+        method = "sock_impl_set_options"
+        params = {"impl_name": "posix", "enable_quickack": True,
+                  "enable_zerocopy_send_server": True,
+                  "enable_zerocopy_send_client": True}
+        return self._request(method, params)
+
     def transport_create_caching(self, trtype):
         params = {
             "trtype": trtype,
@@ -178,7 +184,7 @@ class RPCClient:
                 "trtype": trtype,
                 "adrfam": "IPv4",
                 "traddr": traddr,
-                "trsvcid": trsvcid
+                "trsvcid": str(trsvcid)
             }
         }
         if ana_state:
@@ -280,9 +286,8 @@ class RPCClient:
             "lvs_name": lvs_name,
             "thin_provision": True,
             "clear_method": "unmap",
+            "lvol_priority_class": lvol_priority_class,
         }
-        if lvol_priority_class:
-            params["lvol_priority_class"] = lvol_priority_class
         return self._request("bdev_lvol_create", params)
 
     def delete_lvol(self, name):
@@ -387,7 +392,8 @@ class RPCClient:
         return self._request2("qos_vbdev_delete", params)
 
     def bdev_alceml_create(self, alceml_name, nvme_name, uuid, pba_init_mode=3,
-                           alceml_cpu_mask="", alceml_worker_cpu_mask=""):
+                           alceml_cpu_mask="", alceml_worker_cpu_mask="", pba_page_size=2097152,
+                           write_protection=False):
         params = {
             "name": alceml_name,
             "cntr_path": nvme_name,
@@ -397,7 +403,7 @@ class RPCClient:
             "md_size": 0,
             "use_ram": False,
             "pba_init_mode": pba_init_mode,
-            "pba_page_size": 2097152,
+            "pba_page_size": pba_page_size,
             "uuid": uuid,
             # "use_scheduling": True,
             "use_optimized": True,
@@ -408,11 +414,13 @@ class RPCClient:
             params["bdb_lcpu_mask"] = int(alceml_cpu_mask, 16)
         if alceml_worker_cpu_mask:
             params["bdb_lcpu_mask_alt_workers"] = int(alceml_worker_cpu_mask, 16)
+        if write_protection:
+            params["write_protection"] = True
         return self._request("bdev_alceml_create", params)
 
     def bdev_distrib_create(self, name, vuid, ndcs, npcs, num_blocks, block_size, jm_names,
                             chunk_size, ha_comm_addrs=None, ha_inode_self=None, pba_page_size=2097152,
-                            distrib_cpu_mask="", ha_is_non_leader=True, jm_vuid=0):
+                            distrib_cpu_mask="", ha_is_non_leader=True, jm_vuid=0, write_protection=False):
         """"
             // Optional (not specified = no HA)
             // Comma-separated communication addresses, for each node, e.g. "192.168.10.1:45001,192.168.10.1:32768".
@@ -450,7 +458,8 @@ class RPCClient:
             params['ha_inode_self'] = ha_inode_self
         if distrib_cpu_mask:
             params["bdb_lcpu_mask"] = int(distrib_cpu_mask, 16)
-
+        if write_protection:
+            params["write_protection"] = True
         return self._request("bdev_distrib_create", params)
 
     def bdev_lvol_delete_lvstore(self, name):
@@ -595,8 +604,8 @@ class RPCClient:
     def bdev_nvme_set_options(self):
         params = {
             # "action_on_timeout": "abort",
-            "bdev_retry_count": 3,
-            "transport_retry_count": 5,
+            "bdev_retry_count": 0,
+            "transport_retry_count": 3,
             "ctrlr_loss_timeout_sec": 1,
             "fast_io_fail_timeout_sec": 0,
             "reconnect_delay_sec": 1,
@@ -773,13 +782,6 @@ class RPCClient:
         params = {"name": name, "vuid": vuid}
         return self._request("bdev_jm_unmap_vuid", params)
 
-    def sock_impl_set_options(self):
-        method = "sock_impl_set_options"
-        params = {"impl_name": "posix", "enable_quickack": True,
-                  "enable_zerocopy_send_server": True,
-                  "enable_zerocopy_send_client": True}
-        return self._request(method, params)
-
     def nvmf_set_config(self, poll_groups_mask):
         params = {"poll_groups_mask": poll_groups_mask}
         return self._request("nvmf_set_config", params)
@@ -792,6 +794,9 @@ class RPCClient:
     def thread_get_stats(self):
         return self._request("thread_get_stats")
 
+    def framework_get_reactors(self):
+        return self._request("framework_get_reactors")
+
     def thread_set_cpumask(self, app_thread_process_id, app_thread_mask):
         params = {"id": app_thread_process_id, "cpumask": app_thread_mask}
         return self._request("thread_set_cpumask", params)
@@ -801,8 +806,8 @@ class RPCClient:
             "name": name,
             "storage_ID": storage_ID,
         }
-        # if qos_high_priority:
-        #     params["qos_high_priority"] = qos_high_priority
+        if qos_high_priority:
+            params["qos_high_priority"] = qos_high_priority
         return self._request("distr_migration_to_primary_start", params)
 
     def distr_migration_status(self, name):
@@ -814,16 +819,16 @@ class RPCClient:
             "name": name,
             "storage_ID": storage_ID,
         }
-        # if qos_high_priority:
-        #     params["qos_high_priority"] = qos_high_priority
+        if qos_high_priority:
+            params["qos_high_priority"] = qos_high_priority
         return self._request("distr_migration_failure_start", params)
 
     def distr_migration_expansion_start(self, name, qos_high_priority=False):
         params = {
             "name": name,
         }
-        # if qos_high_priority:
-        #     params["qos_high_priority"] = qos_high_priority
+        if qos_high_priority:
+            params["qos_high_priority"] = qos_high_priority
         return self._request("distr_migration_expansion_start", params)
 
     def bdev_raid_add_base_bdev(self, raid_bdev, base_bdev):
@@ -842,10 +847,7 @@ class RPCClient:
 
     def bdev_lvol_get_lvstores(self, name):
         params = {"lvs_name": name}
-        _, err = self._request2("bdev_lvol_get_lvstores", params)
-        if err:
-            return False
-        return True
+        return self._request("bdev_lvol_get_lvstores", params)
 
     def bdev_lvol_resize(self, name, size_in_mib):
         params = {
@@ -897,7 +899,7 @@ class RPCClient:
                 "trtype": trtype,
                 "adrfam": "IPv4",
                 "traddr": traddr,
-                "trsvcid": trsvcid
+                "trsvcid": str(trsvcid)
             }
         }
         return self._request("nvmf_subsystem_remove_listener", params)
@@ -978,10 +980,31 @@ class RPCClient:
         }
         return self._request("nvmf_set_max_subsystems", params)
 
-    def bdev_lvol_set_lvs_ops(self, lvs_name, groupid, subsystem_port=4420):
+    def bdev_lvol_set_lvs_ops(self, lvs_name, groupid, subsystem_port=9090):
         params = {
             "groupid": groupid,
             "lvs_name": lvs_name,
             "subsystem_port": subsystem_port,
         }
         return self._request("bdev_lvol_set_lvs_op", params)
+
+    def bdev_lvol_get_lvol_delete_status(self, name):
+        """
+        Returns :-
+            0: lvol is deleted.
+            1: lvole deletion is in progress.
+            2: No delete action on lvol or the delete requets is queued or previous delete request
+               failed due to error.
+        """
+        params = {
+            "name": name
+        }
+        return self._request("bdev_lvol_get_lvol_delete_status", params)
+
+    def bdev_lvol_set_lvs_read_only(self, lvs_name, read_only=False):
+        params = {
+            "lvs_name": lvs_name,
+            "read_only": read_only,
+        }
+        return self._request("bdev_lvol_set_lvs_read_only", params)
+
