@@ -1,9 +1,10 @@
 # coding=utf-8
 import time
-from datetime import datetime
+from datetime import datetime, timezone
 
 from simplyblock_core import constants, db_controller, utils
 from simplyblock_core.controllers import tasks_events, tasks_controller
+from simplyblock_core.models.cluster import Cluster
 from simplyblock_core.models.job_schedule import JobSchedule
 
 
@@ -16,6 +17,7 @@ from simplyblock_core.rpc_client import RPCClient
 
 def task_runner(task):
 
+    task = db_controller.get_task_by_id(task.uuid)
     snode = db_controller.get_storage_node_by_id(task.node_id)
     if not snode:
         task.status = JobSchedule.STATUS_DONE
@@ -36,35 +38,38 @@ def task_runner(task):
         task.write_to_db(db_controller.kv_store)
         return False
 
+    cluster = db_controller.get_cluster_by_id(task.cluster_id)
+    if cluster.status not in [Cluster.STATUS_ACTIVE, Cluster.STATUS_DEGRADED, Cluster.STATUS_READONLY]:
+        task.function_result = "cluster is not active, retrying"
+        task.status = JobSchedule.STATUS_SUSPENDED
+        task.retry += 1
+        task.write_to_db(db_controller.kv_store)
+        return False
 
     if task.status in [JobSchedule.STATUS_NEW, JobSchedule.STATUS_SUSPENDED]:
-        all_devs_online = True
         for node in db_controller.get_storage_nodes_by_cluster_id(task.cluster_id):
+            if node.is_secondary_node:  # pass
+                continue
+
+            if node.status == StorageNode.STATUS_ONLINE and node.online_since:
+                try:
+                    diff = datetime.now(timezone.utc) - datetime.fromisoformat(node.online_since)
+                    if diff.total_seconds() < 60:
+                        task.function_result = "node is online < 1 min, retrying"
+                        task.status = JobSchedule.STATUS_SUSPENDED
+                        task.retry += 1
+                        task.write_to_db(db_controller.kv_store)
+                        return False
+                except Exception as e:
+                    logger.error(f"Failed to get online since: {e}")
+
             for dev in node.nvme_devices:
-                if dev.status not in [NVMeDevice.STATUS_ONLINE,
-                                      NVMeDevice.STATUS_FAILED,
-                                      NVMeDevice.STATUS_FAILED_AND_MIGRATED]:
-                    all_devs_online = False
-                    break
-
-        if not all_devs_online:
-            task.function_result = "Some devs are offline, retrying"
-            task.status = JobSchedule.STATUS_SUSPENDED
-            task.retry += 1
-            task.write_to_db(db_controller.kv_store)
-            return False
-
-        if node.online_since:
-            try:
-                diff = datetime.now() - datetime.fromisoformat(node.online_since)
-                if diff.total_seconds() < 60:
-                    task.function_result = "node is online < 1 min, retrying"
+                if dev.status in [NVMeDevice.STATUS_NEW, NVMeDevice.STATUS_UNAVAILABLE]:
+                    task.function_result = f"Some dev status is {dev.status }, retrying"
                     task.status = JobSchedule.STATUS_SUSPENDED
                     task.retry += 1
                     task.write_to_db(db_controller.kv_store)
                     return False
-            except Exception as e:
-                logger.error(f"Failed to get online since: {e}")
 
         task.status = JobSchedule.STATUS_RUNNING
         task.function_result = ""
@@ -97,7 +102,7 @@ def task_runner(task):
         task.function_params['migration'] = {
             "name": distr_name}
         task.write_to_db(db_controller.kv_store)
-        time.sleep(1)
+        # time.sleep(1)
 
     if "migration" in task.function_params:
         mig_info = task.function_params["migration"]
@@ -121,19 +126,22 @@ while True:
         for cl in clusters:
             tasks = db_controller.get_job_tasks(cl.get_id(), reverse=False)
             for task in tasks:
-                delay_seconds = 3
-                if task.function_name == JobSchedule.FN_DEV_MIG:
+                if task.function_name == JobSchedule.FN_DEV_MIG and task.status != JobSchedule.STATUS_DONE:
+                    task = db_controller.get_task_by_id(task.uuid)
                     if task.status in [JobSchedule.STATUS_NEW, JobSchedule.STATUS_SUSPENDED]:
                         active_task = tasks_controller.get_active_node_mig_task(task.cluster_id, task.node_id)
                         if active_task:
                             logger.info("task found on same node, retry")
                             continue
-                    if task.status != JobSchedule.STATUS_DONE:
-                        # get new task object because it could be changed from cancel task
-                        task = db_controller.get_task_by_id(task.uuid)
+
+                        res = task_runner(task)
+                        if res:
+                            tasks_events.task_updated(task)
+
+                    elif task.status == JobSchedule.STATUS_RUNNING:
+
                         res = task_runner(task)
                         if res:
                             tasks_events.task_updated(task)
-                            continue
 
     time.sleep(3)
