@@ -16,7 +16,7 @@ class SNodeClientException(Exception):
 
 class SNodeClient:
 
-    def __init__(self, ip_address, timeout=120, retry=5):
+    def __init__(self, ip_address, timeout=300, retry=5):
         self.ip_address = ip_address
         self.url = 'http://%s/snode/' % self.ip_address
         self.timeout = timeout
@@ -28,15 +28,21 @@ class SNodeClient:
         self.session.mount("http://", HTTPAdapter(max_retries=retries))
         self.session.timeout = self.timeout
 
-    def _request(self, method, path, params=None):
+    def _request(self, method, path, payload=None):
         try:
-            logger.debug("Requesting path: %s, params: %s", path, params)
-            dt = None
-            if params:
-                dt = json.dumps(params)
-            response = self.session.request(method, self.url+path, data=dt, timeout=self.timeout)
+            logger.debug("Requesting path: %s, params: %s", path, payload)
+            data = None
+            params = None
+            if payload:
+                if method == "GET" :
+                    params = payload
+                else:
+                    data = json.dumps(payload)
+
+            response = self.session.request(method, self.url+path, data=data,
+                                            timeout=self.timeout, params=params)
         except Exception as e:
-            return None, str(e)
+            raise e
 
         logger.debug("Response: status_code: %s, content: %s",
                      response.status_code, response.content)
@@ -72,7 +78,8 @@ class SNodeClient:
 
     def spdk_process_start(self, spdk_cpu_mask, spdk_mem, spdk_image=None, spdk_debug=None, cluster_ip=None,
                            fdb_connection=None, namespace=None, server_ip=None, rpc_port=None,
-                           rpc_username=None, rpc_password=None, multi_threading_enabled=False, timeout=0):
+                           rpc_username=None, rpc_password=None, multi_threading_enabled=False, timeout=0, ssd_pcie=None,
+                           total_mem=None):
         params = {
             "cluster_ip": cluster_ip,
             "server_ip": server_ip,
@@ -96,6 +103,10 @@ class SNodeClient:
             params["multi_threading_enabled"] = multi_threading_enabled
         if timeout:
             params["timeout"] = timeout
+        if ssd_pcie:
+            params["ssd_pcie"] = ssd_pcie
+        if total_mem:
+            params["total_mem"] = total_mem
         return self._request("POST", "spdk_process_start", params)
 
     def join_swarm(self, cluster_ip, join_token, db_connection, cluster_id):
@@ -106,8 +117,12 @@ class SNodeClient:
             "db_connection": db_connection}
         return self._request("POST", "join_swarm", params)
 
-    def spdk_process_kill(self):
-        return self._request("GET", "spdk_process_kill")
+    def spdk_process_kill(self, rpc_port):
+        params = {"rpc_port": rpc_port}
+        try:
+            return self._request("GET", "spdk_process_kill", params)
+        except Exception as e:
+            return False
 
     def leave_swarm(self):
         return self._request("GET", "leave_swarm")
@@ -129,8 +144,23 @@ class SNodeClient:
         params = {"device_pci": device_pci}
         return self._request("POST", "bind_device_to_spdk", params)
 
-    def spdk_process_is_up(self):
-        return self._request("GET", "spdk_process_is_up")
+    def spdk_process_is_up(self, rpc_port):
+        params = {"rpc_port": rpc_port}
+        return self._request("GET", "spdk_process_is_up", params)
 
     def get_file_content(self, file_name):
         return self._request("GET", f"get_file_content/{file_name}")
+
+
+    def firewall_set_port(self, port_id, port_type="tcp", action="block", rpc_port=None):
+        params = {
+            "port_id": port_id,
+            "port_type": port_type,
+            "action": action,
+            "rpc_port": rpc_port,
+        }
+        return self._request("POST", "firewall_set_port", params)
+
+    def get_firewall(self,rpc_port=None):
+        params = {"rpc_port": rpc_port}
+        return self._request("GET", "get_firewall", params)
