@@ -3,11 +3,12 @@ import json
 import logging
 import math
 import os
-import psutil
 import random
 import re
 import string
+import subprocess
 import sys
+from typing import Union
 
 import docker
 from prettytable import PrettyTable
@@ -17,6 +18,15 @@ from simplyblock_core import constants
 from simplyblock_core import shell_utils
 from simplyblock_core.models.job_schedule import JobSchedule
 
+CONFIG_KEYS = [
+    "app_thread_core",
+    "jm_cpu_core",
+    "poller_cpu_cores",
+    "alceml_cpu_cores",
+    "alceml_worker_cpu_cores",
+    "distrib_cpu_cores",
+    "jc_singleton_core",
+]
 
 def get_env_var(name, default=None, is_required=False):
     if not name:
@@ -96,26 +106,27 @@ def print_table(data: list, title=None):
         return x.__str__()
 
 
-def humanbytes(B):
-    """Return the given bytes as a human friendly KB, MB, GB, or TB string."""
-    if not B:
-        return "0"
-    B = float(B)
-    KB = float(1000)
-    MB = float(KB ** 2) # 1,048,576
-    GB = float(KB ** 3) # 1,073,741,824
-    TB = float(KB ** 4) # 1,099,511,627,776
+_humanbytes_parameter = {
+    'si': (10, 3, math.log10, ''),
+    'iec': (2, 10, math.log2, 'i'),
+    'jedec': (2, 10, math.log2, ''),
+}
 
-    if B < KB:
-        return '{0} {1}'.format(B, 'Bytes' if 0 == B > 1 else 'Byte')
-    elif KB <= B < MB:
-        return '{0:.1f} KB'.format(B / KB)
-    elif MB <= B < GB:
-        return '{0:.1f} MB'.format(B / MB)
-    elif GB <= B < TB:
-        return '{0:.1f} GB'.format(B / GB)
-    elif TB <= B:
-        return '{0:.1f} TB'.format(B / TB)
+
+def humanbytes(size: int, mode: str = 'si') -> str:
+    """Return the given bytes as a human friendly including the appropriate unit."""
+    if not size:
+        return '0 B'
+
+    base, exponent, log, infix = _humanbytes_parameter[mode]
+
+    prefixes = ['', 'k' if mode == 'si' else 'K', 'M', 'G', 'T', 'P', 'E', 'Z']
+    exponent_multiplier = min(int(log(size) / exponent), len(prefixes) - 1)
+
+    size_in_unit = size / (base ** (exponent * exponent_multiplier))
+    prefix = prefixes[exponent_multiplier]
+
+    return f"{size_in_unit:.1f} {prefix}{infix if prefix else ''}B"
 
 
 def generate_string(length):
@@ -322,40 +333,6 @@ def get_random_vuid():
     return r
 
 
-def calculate_core_allocation(cpu_count):
-    '''
-    If number of cpu cores >= 8, tune cpu core mask
-        1. Never use core 0 for spdk.
-        2. Core 1 is for app_thread
-        3. Core 2 for Journal manager
-        4. Poller cpu cores are 30% of Available cores
-        5. Alceml cpu cores are 30% of Available cores
-        6. Distribs cpu cores are 40% of Available cores
-    JIRA ticket link/s
-    https://simplyblock.atlassian.net/browse/SFAM-885
-    '''
-
-    if cpu_count > 64:
-        cpu_count = 64
-
-    all_cores = list(range(0, cpu_count))
-    app_thread_core = all_cores[1:2]
-    jm_cpu_core = all_cores[2:3]
-
-    # Calculate available cores
-    available_cores_count = cpu_count - 3
-
-    # Calculate cpus counts
-    poller_cpus_count = int(available_cores_count * 0.3)
-    alceml_cpus_cout = int(available_cores_count * 0.3)
-
-    # Calculate cpus cores
-    poller_cpu_cores = all_cores[3:poller_cpus_count+3]
-    alceml_cpu_cores = all_cores[3+poller_cpus_count:poller_cpus_count+alceml_cpus_cout+3]
-    distrib_cpu_cores = all_cores[3+poller_cpus_count+alceml_cpus_cout:]
-
-    return app_thread_core, jm_cpu_core, poller_cpu_cores, alceml_cpu_cores, distrib_cpu_cores
-
 def hexa_to_cpu_list(cpu_mask):
     # Convert the hex string to an integer
     mask_int = int(cpu_mask, 16)
@@ -377,66 +354,60 @@ def hexa_to_cpu_list(cpu_mask):
     return cpu_list
 
 def calculate_core_allocation(cpu_cores):
-
-    '''
-    Calculation is according to mask until 24 vcpu table
-    https://docs.google.com/spreadsheets/d/14vxL9W31-7oFT6tEeLNuaJJO0FwG9cZK/edit?gid=1468847899#gid=1468847899
-    '''
-
     if len(cpu_cores) >= 23:
-        app_thread_core = [cpu_cores[pos - 1] for pos in [16]]
-        jm_cpu_core = [cpu_cores[pos - 1] for pos in [17, 22]]
-        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [6, 7, 8, 14, 15, 18, 19, 20]]
+        app_thread_core = [cpu_cores[pos - 1] for pos in [10]]
+        jm_cpu_core = [cpu_cores[pos - 1] for pos in [11, 22]]
+        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [9, 7, 8, 12, 14, 15, 19, 20]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1, 2, 3]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [13, 21]]
-        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 5, 9, 10, 11, 12]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [13, 21]]
+        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 5, 6, 16, 17, 18]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [23]]
     elif len(cpu_cores) >= 21:
         app_thread_core = [cpu_cores[pos - 1] for pos in [14]]
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [15, 21]]
-        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 5, 6, 7, 13, 16, 17]]
+        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 7, 8, 9, 10, 11, 13]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1, 2]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [12, 20]]
-        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 8, 9, 10, 11, 19]]
-        jc_singleton_core = [cpu_cores[pos - 1] for pos in [18]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [12, 20]]
+        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 5, 6, 16, 17, 18]]
+        jc_singleton_core = [cpu_cores[pos - 1] for pos in [19]]
     elif len(cpu_cores) >= 19:
         app_thread_core = [cpu_cores[pos - 1] for pos in [13]]
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [14]]
-        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [5, 6, 7, 12, 15, 16]]
+        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 8, 7, 9, 12, 15]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1, 2]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [11, 19]]
-        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 8, 9, 10, 18]]
-        jc_singleton_core = [cpu_cores[pos - 1] for pos in [17]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [11, 19]]
+        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 5, 6, 16, 17, 18]]
+        jc_singleton_core = [cpu_cores[pos - 1] for pos in [10]]
     elif len(cpu_cores) >= 17:
         app_thread_core = [cpu_cores[pos - 1] for pos in [12]]
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [13]]
-        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 5, 6, 11, 14]]
+        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [6, 7, 8, 11, 14]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1, 2]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [10]]
-        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 7, 8, 9, 16, 17]]
-        jc_singleton_core = [cpu_cores[pos - 1] for pos in [15]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [10]]
+        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 5, 15, 16, 17]]
+        jc_singleton_core = [cpu_cores[pos - 1] for pos in [9]]
     elif len(cpu_cores) >= 15:
-        app_thread_core = [cpu_cores[pos - 1] for pos in [11]]
-        jm_cpu_core = [cpu_cores[pos - 1] for pos in [12]]
-        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [5, 6, 10, 13]]
+        app_thread_core = [cpu_cores[pos - 1] for pos in [8]]
+        jm_cpu_core = [cpu_cores[pos - 1] for pos in [15]]
+        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [6, 7, 10, 13]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1, 2]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [9]]
-        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 7, 8, 15]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [9]]
+        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 5, 11, 12]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [14]]
     elif len(cpu_cores) >= 13:
         app_thread_core = [cpu_cores[pos - 1] for pos in [9]]
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [10]]
-        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 5, 11]]
+        poller_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 5, 6, 7]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [8]]
-        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 6, 7, 13]]
-        jc_singleton_core = [cpu_cores[pos - 1] for pos in [12]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [8]]
+        distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 11, 12]]
+        jc_singleton_core = [cpu_cores[pos - 1] for pos in [13]]
     elif len(cpu_cores) >= 11:
         app_thread_core = [cpu_cores[pos - 1] for pos in [8]]
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [9]]
         poller_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4, 10]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [7]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [7]]
         distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 5, 6]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [11]]
     elif len(cpu_cores) >= 9:
@@ -444,7 +415,7 @@ def calculate_core_allocation(cpu_cores):
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [8]]
         poller_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 4]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [4, 9]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [4, 9]]
         distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 5, 6]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [7]]
     elif len(cpu_cores) >= 7:
@@ -452,7 +423,7 @@ def calculate_core_allocation(cpu_cores):
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [5]]
         poller_cpu_cores = [cpu_cores[pos - 1] for pos in [3, 5]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [1]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
         distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 4]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [7]]
     elif len(cpu_cores) >= 5:
@@ -460,7 +431,7 @@ def calculate_core_allocation(cpu_cores):
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [4]]
         poller_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 3]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [1]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
         distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 3]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [5]]
     elif len(cpu_cores) >= 4:
@@ -468,15 +439,28 @@ def calculate_core_allocation(cpu_cores):
         jm_cpu_core = [cpu_cores[pos - 1] for pos in [4]]
         poller_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 3]]
         alceml_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
-        alceml_cpu_worker_cores = [cpu_cores[pos - 1] for pos in [1]]
+        alceml_worker_cpu_cores = [cpu_cores[pos - 1] for pos in [1]]
         distrib_cpu_cores = [cpu_cores[pos - 1] for pos in [2, 3]]
         jc_singleton_core = [cpu_cores[pos - 1] for pos in [4]]
     else:
-        app_thread_core = jm_cpu_core = poller_cpu_cores = alceml_cpu_cores = alceml_cpu_worker_cores = distrib_cpu_cores = jc_singleton_core = []
+        app_thread_core = jm_cpu_core = poller_cpu_cores = alceml_cpu_cores = alceml_worker_cpu_cores = distrib_cpu_cores = jc_singleton_core = []
 
-    return app_thread_core, jm_cpu_core, poller_cpu_cores, alceml_cpu_cores, alceml_cpu_worker_cores, distrib_cpu_cores, jc_singleton_core
+    return app_thread_core, jm_cpu_core, poller_cpu_cores, alceml_cpu_cores, alceml_worker_cpu_cores, distrib_cpu_cores, jc_singleton_core
 
 
+def isolate_cores(spdk_cpu_mask):
+    spdk_cores = hexa_to_cpu_list(spdk_cpu_mask)
+    hyperthreading_enabled = is_hyperthreading_enabled_via_siblings()
+    isolated_cores = spdk_cores
+    siblings = parse_thread_siblings()
+    isolated_full = set(isolated_cores)
+    if hyperthreading_enabled:
+        for cpu in siblings[0]:
+            isolated_full.discard(cpu)
+    else:
+        isolated_full.discard(0)
+    return isolated_full
+
 def generate_mask(cores):
     mask = 0
     for core in cores:
@@ -511,7 +495,7 @@ def calculate_pool_count(alceml_count, number_of_distribs, cpu_count, poller_cou
     small_pool_count = 384 * (alceml_count + number_of_distribs + 3 + poller_count) + (6 + alceml_count + number_of_distribs) * 256 + poller_number * 127 + 384 + 128 * poller_number + constants.EXTRA_SMALL_POOL_COUNT
     #large_pool_count = (3 + alceml_count + lvol_count + 2 * snap_count + 1) * 32 + poller_number * 15 + 384 + 16 * poller_number + constants.EXTRA_LARGE_POOL_COUNT
     large_pool_count = 48 * (alceml_count + number_of_distribs + 3 + poller_count) + (6 + alceml_count + number_of_distribs) * 32 + poller_number * 15 + 384 + 16 * poller_number + constants.EXTRA_LARGE_POOL_COUNT
-    return small_pool_count, large_pool_count
+    return 2*small_pool_count, 2*large_pool_count
 
 
 def calculate_minimum_hp_memory(small_pool_count, large_pool_count, lvol_count, max_prov, cpu_count):
@@ -523,14 +507,12 @@ def calculate_minimum_hp_memory(small_pool_count, large_pool_count, lvol_count,
     return: minimum_hp_memory in bytes
     '''
     pool_consumption = (small_pool_count * 8 + large_pool_count * 128) / 1024 + 1092
-    max_prov_tb = max_prov / (1024 * 1024 * 1024 * 1024)
-    memory_consumption = (4 * cpu_count + 1.0277 * pool_consumption + 25 * lvol_count) * (1024 * 1024) + (250 * 1024 * 1024) * 1.1 * max_prov_tb + constants.EXTRA_HUGE_PAGE_MEMORY
+    memory_consumption = (4 * cpu_count + 1.0277 * pool_consumption + 25 * lvol_count) * (1024 * 1024) + (250 * 1024 * 1024) * 1.1 * convert_size(max_prov, 'TiB') + constants.EXTRA_HUGE_PAGE_MEMORY
     return int(memory_consumption)
 
 
 def calculate_minimum_sys_memory(max_prov, total):
-    max_prov_tb = max_prov / (1024 * 1024 * 1024 * 1024)
-    minimum_sys_memory = (250 * 1024 * 1024) * 1.1 * max_prov_tb + (constants.EXTRA_SYS_MEMORY * total)
+    minimum_sys_memory = (250 * 1024 * 1024) * 1.1 * convert_size(max_prov, 'TiB') + (constants.EXTRA_SYS_MEMORY * total)
     logger.debug(f"Minimum system memory is {humanbytes(minimum_sys_memory)}")
     return int(minimum_sys_memory)
 
@@ -610,40 +592,73 @@ def get_logger(name=""):
     return logg
 
 
-def parse_size(size_string: str):
+def _parse_unit(unit: str, mode: str = 'si/iec', strict: bool = True) -> tuple[int, int]:
+    """Parse the given unit, returning the associated base and exponent
+
+    Mode can be either 'si/iec' to parse decimal (SI) and binary (IEC) units, or
+    'jedec' for binary only units. If `strict`, parsing will be case-sensitive and
+    expect the 'B' suffix.
+    """
+    regexes = {
+        'si/iec': r'^((?P<prefix>[kKMGTPEZ])(?P<binary>i)?)?' + ('B$' if strict else 'B?$'),
+        'jedec': r'^(?P<prefix>[KMGTPEZ])?' + ('B$' if strict else 'B?$'),
+    }
+
+    m = re.match(regexes[mode], unit, flags=re.IGNORECASE if not strict else 0)
+    if m is None:
+        raise ValueError("Invalid unit")
+
+    binary = (mode == 'jedec') or (m.group('binary') is not None)
+    prefix = m.group('prefix') or ''
+
+    if strict and (binary and (prefix == 'k')) or ((not binary) and (prefix == 'K')):
+        raise ValueError("Invalid unit")
+
+    exponent_multipliers = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']
+    return (
+        2 if binary else 10,
+        (10 if binary else 3) * exponent_multipliers.index(prefix.upper())
+    )
+
+
+def parse_size(size_string: str, mode: str = 'si/iec', unit: str = '', strict: bool = False) -> int:
+    """Parse the given data size
+
+    If passed and not explicitly given, 'unit' will be assumed.
+    Mode can be either 'si/iec' to parse decimal (SI) and binary (IEC) units, or
+    'jedec' for binary only units. If `strict`, parsing will be case-sensitive and
+    expect the 'B' suffix.
+    """
+    if not unit:
+        try:
+            x = int(size_string)
+            return x
+        except Exception:
+            pass
+
     try:
-        x = int(size_string)
-        return x
-    except Exception:
-        pass
-    try:
-        if size_string:
-            size_string = size_string.lower()
-            size_string = size_string.replace(" ", "")
-            size_string = size_string.replace("b", "")
-            size_number = int(size_string[:-1])
-            size_v = size_string[-1]
-            one_k = 1000
-            multi = 0
-            if size_v == "k":
-                multi = 1
-            elif size_v == "m":
-                multi = 2
-            elif size_v == "g":
-                multi = 3
-            elif size_v == "t":
-                multi = 4
-            else:
-                print(f"Error parsing size: {size_string}")
-                return -1
-            return size_number * math.pow(one_k, multi)
-        else:
-            return -1
-    except:
-        print(f"Error parsing size: {size_string}")
+        m = re.match(r'^(?P<size_in_unit>\d+) ?(?P<unit>\w+)?$', size_string.strip())
+        if m is None:
+            raise ValueError(f"Invalid size: {size_string}")
+
+        size_in_unit = int(m.group('size_in_unit'))
+        unit = m.group('unit') if m.group('unit') else unit
+        base, exponent = _parse_unit(unit, mode, strict=strict)
+        return size_in_unit * (base ** exponent)
+    except ValueError:
         return -1
 
 
+def convert_size(size: int, unit: str) -> int:
+    """Convert the given number of bytes to target unit
+
+    Accepts both decimal (kB, MB, ...) and binary (KiB, MiB, ...) units.
+    Note that the result will be cast to int, i.e. rounded down.
+    """
+    base, exponent = _parse_unit(unit, 'si/iec')
+    return int(size / (base ** exponent))
+
+
 def nearest_upper_power_of_2(n):
     # Check if n is already a power of 2
     if (n & (n - 1)) == 0:
@@ -689,6 +704,7 @@ def handle_task_result(task: JobSchedule, res: dict, allowed_error_codes = None)
                 task.status = JobSchedule.STATUS_SUSPENDED
                 del task.function_params['migration']
 
+            task.status = JobSchedule.STATUS_DONE
             task.write_to_db()
             return True
 
@@ -714,3 +730,202 @@ def handle_task_result(task: JobSchedule, res: dict, allowed_error_codes = None)
 
 
 logger = get_logger(__name__)
+
+
+def get_next_port(cluster_id):
+    from simplyblock_core.db_controller import DBController
+    db_controller = DBController()
+
+    port = constants.LVOL_NVMF_PORT_START
+    used_ports = []
+    for node in db_controller.get_storage_nodes_by_cluster_id(cluster_id):
+        if node.lvol_subsys_port > 0:
+            used_ports.append(node.lvol_subsys_port)
+
+    next_port = port
+    while True:
+        if next_port not in used_ports:
+            return next_port
+        next_port += 1
+
+def get_next_rpc_port(cluster_id):
+    from simplyblock_core.db_controller import DBController
+    db_controller = DBController()
+
+    port = 8080
+    used_ports = []
+    for node in db_controller.get_storage_nodes_by_cluster_id(cluster_id):
+        if node.rpc_port > 0:
+            used_ports.append(node.rpc_port)
+
+    for i in range(1000):
+        next_port = port + i
+
+        if next_port not in used_ports:
+            return next_port
+
+    return 0
+
+def get_next_dev_port(cluster_id):
+    from simplyblock_core.db_controller import DBController
+    db_controller = DBController()
+
+    port = 9080
+    used_ports = []
+    for node in db_controller.get_storage_nodes_by_cluster_id(cluster_id):
+        if node.nvmf_port > 0:
+            used_ports.append(node.nvmf_port)
+
+    for i in range(1000):
+        next_port = port + i
+
+    return port
+
+
+def generate_realtime_variables_file(isolated_cores, realtime_variables_file_path="/etc/tuned/realtime-variables.conf"):
+    """
+    Generate or update the realtime-variables.conf file.
+    Args:
+        isolated_cores (set): set of isolated cores to write.
+        file_path (str): Path to the file.
+    """
+    # Ensure the directory exists
+    core_list = ",".join(map(str, isolated_cores))
+    tuned_dir = "/etc/tuned/realtime"
+    os.makedirs(tuned_dir, exist_ok=True)
+
+    # Create tuned.conf
+    tuned_conf_content = f"""[main]
+include=latency-performance
+[bootloader]
+cmdline_add=isolcpus={core_list} nohz_full={core_list} rcu_nocbs={core_list}
+"""
+
+    tuned_conf_path = f"{tuned_dir}/tuned.conf"
+    with open(tuned_conf_path, "w") as f:
+        f.write(tuned_conf_content)
+    content = f"isolated_cores={core_list}\n"
+    try:
+        process = subprocess.run(
+            ["sudo", "tee", realtime_variables_file_path],
+            input=content.encode("utf-8"),
+            stdout=subprocess.DEVNULL,  # Suppress standard output
+            stderr=subprocess.PIPE,  # Capture standard error
+            check=True  # Raise an error if command fails
+        )
+        logger.info(f"Successfully wrote to {realtime_variables_file_path}")
+    except subprocess.CalledProcessError as e:
+        logger.error(f"Error writing to file: {e}")
+
+def run_tuned():
+    try:
+        subprocess.run(
+            ["sudo", "tuned-adm", "profile", "realtime"],
+            check=True
+        )
+        print(f"Successfully run the tuned adm profile")
+    except subprocess.CalledProcessError as e:
+        logger.error(f"Error running the tuned adm profile: {e}")
+
+
+def parse_thread_siblings():
+    """Parse the thread siblings from the sysfs topology."""
+    siblings = {}
+    for cpu in os.listdir("/sys/devices/system/cpu/"):
+        if cpu.startswith("cpu") and cpu[3:].isdigit():
+            cpu_id = int(cpu[3:])
+            try:
+                with open(f"/sys/devices/system/cpu/{cpu}/topology/thread_siblings_list") as f:
+                    siblings[cpu_id] = [
+                        int(x) for x in f.read().strip().split(",")
+                    ]
+            except FileNotFoundError:
+                siblings[cpu_id] = [cpu_id]  # No siblings for this CPU
+    return siblings
+
+
+def is_hyperthreading_enabled_via_siblings():
+    """
+    Check if hyperthreading is enabled based on thread_siblings_list.
+    """
+    siblings = parse_thread_siblings()
+    for sibling_list in siblings.values():
+       if len(sibling_list) > 1:
+            return True
+    return False
+
+
+def load_core_distribution_from_file(file_path, number_of_cores):
+    """Load core distribution from the configuration file or use default."""
+    # Check if the file exists
+    if not os.path.exists(file_path):
+        logger.warning("Configuration file not found. Using default values.")
+        return None  # Indicate that defaults should be used
+
+    # Attempt to read the file
+    try:
+        with open(file_path, "r") as configfile:
+            config = {}
+            for line in configfile:
+                if line.strip() and not line.startswith("#"):  # Skip comments and empty lines
+                    key, value = line.split("=", 1)
+                    key = key.strip()
+                    value = value.strip()
+                    if key in CONFIG_KEYS:
+                        config[key] = [int(core) for core in value.split(",")] if value else None
+                        if any(core > number_of_cores for core in config[key]):
+                            raise ValueError(f"Invalid distribution, the index of the core {value}: "
+                                             f"must be in range of number of cores {number_of_cores}")
+
+
+            # Validate all keys are present
+            if not all(key in config and config[key] is not None for key in CONFIG_KEYS):
+                logger.warning("Incomplete configuration provided. Using default values.")
+                return None  # Indicate that defaults should be used
+
+            return config
+    except Exception as e:
+        logger.warning(f"Error reading configuration file: {e}, Using default values.")
+        return None  # Indicate that defaults should be used
+
+
+def store_cores_config(spdk_cpu_mask):
+    cores_config = {
+        "cpu_mask": spdk_cpu_mask
+    }
+    file_path = constants.TEMP_CORES_FILE
+    # Ensure the directory exists
+    os.makedirs(os.path.dirname(file_path), exist_ok=True)
+    cores_config_json = json.dumps(cores_config, indent=4)
+
+    # Write the dictionary to the JSON file
+    try:
+        process = subprocess.run(
+            ["sudo", "tee", file_path],
+            input=cores_config_json.encode("utf-8"),
+            stdout=subprocess.DEVNULL,  # Suppress standard output
+            stderr=subprocess.PIPE,  # Capture standard error
+            check=True  # Raise an error if command fails
+        )
+        logger.info(f"JSON file successfully written to {file_path}")
+    except subprocess.CalledProcessError as e:
+        logger.error(f"Error writing to file: {e}")
+
+def init_sentry_sdk(name=None):
+    # import sentry_sdk
+    # params = {
+    #     "dsn": constants.SENTRY_SDK_DNS,
+    #     # Set traces_sample_rate to 1.0 to capture 100%
+    #     # of transactions for tracing.
+    #     "traces_sample_rate": 1.0,
+    #     # Add request headers and IP for users,
+    #     # see https://docs.sentry.io/platforms/python/data-management/data-collected/ for more info
+    #     "send_default_pii": True,
+    # }
+    # if name:
+    #     params["server_name"] = name
+    # sentry_sdk.init(**params)
+    # # from sentry_sdk import set_level
+    # # set_level("critical")
+
+    return True
