@@ -8,7 +8,7 @@ from flask import Blueprint, request
 
 
 from simplyblock_web import utils
-from simplyblock_core import db_controller
+from simplyblock_core import db_controller, utils as core_utils
 from simplyblock_core.controllers import caching_node_controller
 
 logger = logging.getLogger(__name__)
@@ -33,7 +33,6 @@ def add_node_to_cluster():
 
     data_nics_list = []
     spdk_cpu_mask = None
-    spdk_mem = None
     spdk_image = None
     namespace = None
     multipathing = True
@@ -41,11 +40,9 @@ def add_node_to_cluster():
     if 'spdk_cpu_mask' in cl_data:
         spdk_cpu_mask = cl_data['spdk_cpu_mask']
 
-    if 'spdk_mem' in cl_data:
-        mem = cl_data['spdk_mem']
-        spdk_mem = utils.parse_size(mem)
-        if spdk_mem < 1 * 1024 * 1024:
-            return utils.get_response_error(f"SPDK memory:{mem} must be larger than 1G", 400)
+    spdk_mem = core_utils.parse_size(cl_data['spdk_mem']) if 'spdk_mem' in cl_data else None
+    if spdk_mem is not None and spdk_mem < core_utils.parse_size('1GiB'):
+        return utils.get_response_error(f"SPDK memory:{spdk_mem} must be larger than 1G", 400)
 
     if 'spdk_image' in cl_data:
         spdk_image = cl_data['spdk_image']
