@@ -12,7 +12,7 @@ logger_handler = logging.StreamHandler()
 logger_handler.setFormatter(logging.Formatter('%(asctime)s: %(levelname)s: %(message)s'))
 logger = logging.getLogger()
 logger.addHandler(logger_handler)
-logger.setLevel(constants.LOG_LEVEL)
+logger.setLevel(logging.DEBUG)
 
 
 app = Flask(__name__)
@@ -53,4 +53,8 @@ if __name__ == '__main__':
         from blueprints import snode_ops
         app.register_blueprint(snode_ops.bp)
 
+    if mode == "storage_node_k8s":
+        from blueprints import snode_ops_k8s
+        app.register_blueprint(snode_ops_k8s.bp)
+
     app.run(host='0.0.0.0', debug=constants.LOG_WEB_DEBUG)
